# Generated by Django 2.2.8 on 2020-02-13 20:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0193_jednostka_dla_punktacji_dyscyplin"),
    ]

    operations = [
        migrations.DeleteModel(name="Sumy_Patent_View",),
        migrations.DeleteModel(name="Sumy_Praca_Doktorska_View",),
        migrations.DeleteModel(name="Sumy_Praca_Habilitacyjna_View",),
        migrations.DeleteModel(name="Sumy_View",),
        migrations.DeleteModel(name="Sumy_Wydawnictwo_Ciagle_View",),
        migrations.DeleteModel(name="Sumy_Wydawnictwo_Zwarte_View",),
    ]



# Generated by Django 3.0.14 on 2021-07-13 23:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0017_publikacjainstytucji_usertype"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="oswiadczenieinstytucji",
            options={
                "verbose_name": "Oświadczenie instytucji",
                "verbose_name_plural": "Oświadczenia instytucji",
            },
        ),
        migrations.AlterModelOptions(
            name="publikacjainstytucji",
            options={
                "verbose_name": "Publikacja instytucji",
                "verbose_name_plural": "Publikacje instytucji",
            },
        ),
        migrations.AlterUniqueTogether(
            name="oswiadczenieinstytucji",
            unique_together={("publicationId", "institutionId", "personId")},
        ),
    ]



# Generated by Django 3.0.14 on 2021-06-05 19:28

from django.db import migrations

import bpp.models.dyscyplina_naukowa


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0271_auto_20210530_0225"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dyscyplina_naukowa",
            name="kod",
            field=bpp.models.dyscyplina_naukowa.KodDyscyplinyField(
                max_length=20,
                unique=True,
                validators=[
                    bpp.models.dyscyplina_naukowa.waliduj_format_kodu_numer,
                    bpp.models.dyscyplina_naukowa.waliduj_format_kodu_numer,
                    bpp.models.dyscyplina_naukowa.waliduj_format_kodu_numer,
                ],
            ),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Autorzy',
            fields=[
            ],
            options={
                'db_table': 'bpp_autorzy_mat',
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='AutorzyView',
            fields=[
            ],
            options={
                'db_table': 'bpp_autorzy',
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Kronika_Patent_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Kronika_Praca_Doktorska_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Kronika_Praca_Habilitacyjna_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Kronika_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Kronika_Wydawnictwo_Ciagle_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Kronika_Wydawnictwo_Zwarte_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Rekord',
            fields=[
            ],
            options={
                'ordering': ['tytul_oryginalny_sort'],
                'db_table': 'bpp_rekord_mat',
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Sumy_Patent_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Sumy_Praca_Doktorska_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Sumy_Praca_Habilitacyjna_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Sumy_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Sumy_Wydawnictwo_Ciagle_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Sumy_Wydawnictwo_Zwarte_View',
            fields=[
            ],
            options={
                'managed': False,
            },
            bases=(models.Model,),
        ),
        migrations.AlterModelOptions(
            name='autor_jednostka',
            options={'ordering': ['autor__nazwisko', 'jednostka__nazwa', 'rozpoczal_prace'], 'verbose_name': 'powi\u0105zanie autor-jednostka', 'verbose_name_plural': 'powi\u0105zania autor-jednostka'},
        ),
        migrations.AlterModelOptions(
            name='jezyk',
            options={'ordering': ['nazwa'], 'verbose_name': 'j\u0119zyk', 'verbose_name_plural': 'j\u0119zyki'},
        ),
        migrations.AlterModelOptions(
            name='patent_autor',
            options={'ordering': ('kolejnosc',), 'verbose_name': 'powi\u0105zanie autora z patentem', 'verbose_name_plural': 'powi\u0105zania autor\xf3w z patentami'},
        ),
        migrations.AlterModelOptions(
            name='plec',
            options={'verbose_name': 'p\u0142e\u0107', 'verbose_name_plural': 'p\u0142cie'},
        ),
        migrations.AlterModelOptions(
            name='publikacja_habilitacyjna',
            options={'ordering': ('kolejnosc',), 'verbose_name': 'powi\u0105zanie publikacji z habilitacj\u0105', 'verbose_name_plural': 'powi\u0105zania publikacji z habilitacj\u0105'},
        ),
        migrations.AlterModelOptions(
            name='punktacja_zrodla',
            options={'ordering': ['zrodlo__nazwa', 'rok'], 'verbose_name': 'punktacja \u017ar\xf3d\u0142a', 'verbose_name_plural': 'punktacja \u017ar\xf3d\u0142a'},
        ),
        migrations.AlterModelOptions(
            name='redakcja_zrodla',
            options={'verbose_name': 'redaktor \u017ar\xf3d\u0142a', 'verbose_name_plural': 'redaktorzy \u017ar\xf3d\u0142a'},
        ),
        migrations.AlterModelOptions(
            name='rodzaj_zrodla',
            options={'verbose_name': 'rodzaj \u017ar\xf3d\u0142a', 'verbose_name_plural': 'rodzaje \u017ar\xf3de\u0142'},
        ),
        migrations.AlterModelOptions(
            name='typ_odpowiedzialnosci',
            options={'ordering': ['nazwa'], 'verbose_name': 'typ odpowiedzialno\u015bci autora', 'verbose_name_plural': 'typy odpowiedzialno\u015bci autor\xf3w'},
        ),
        migrations.AlterModelOptions(
            name='tytul',
            options={'verbose_name': 'tytu\u0142', 'verbose_name_plural': 'tytu\u0142y'},
        ),
        migrations.AlterModelOptions(
            name='wydawnictwo_ciagle',
            options={'verbose_name': 'wydawnictwo ci\u0105g\u0142e', 'verbose_name_plural': 'wydawnictwa ci\u0105g\u0142e'},
        ),
        migrations.AlterModelOptions(
            name='wydawnictwo_ciagle_autor',
            options={'ordering': ('kolejnosc',), 'verbose_name': 'powi\u0105zanie autora z wyd. ci\u0105g\u0142ym', 'verbose_name_plural': 'powi\u0105zania autor\xf3w z wyd. ci\u0105g\u0142ymi'},
        ),
        migrations.AlterModelOptions(
            name='wydawnictwo_zwarte_autor',
            options={'ordering': ('kolejnosc',), 'verbose_name': 'powi\u0105zanie autora z wyd. zwartym', 'verbose_name_plural': 'powi\u0105zania autor\xf3w z wyd. zwartymi'},
        ),
        migrations.AlterModelOptions(
            name='wydzial',
            options={'ordering': ['kolejnosc', 'skrot'], 'verbose_name': 'wydzia\u0142', 'verbose_name_plural': 'wydzia\u0142y'},
        ),
        migrations.AlterModelOptions(
            name='zasieg_zrodla',
            options={'verbose_name': 'zasi\u0119g \u017ar\xf3d\u0142a', 'verbose_name_plural': 'zasi\u0119g \u017ar\xf3de\u0142'},
        ),
        migrations.AlterModelOptions(
            name='zrodlo',
            options={'ordering': ['nazwa'], 'verbose_name': '\u017ar\xf3d\u0142o', 'verbose_name_plural': '\u017ar\xf3d\u0142a'},
        ),
        migrations.AlterModelOptions(
            name='zrodlo_informacji',
            options={'verbose_name': '\u017ar\xf3d\u0142o informacji o bibliografii', 'verbose_name_plural': '\u017ar\xf3d\u0142a informacji o bibliografii'},
        ),
        migrations.AddField(
            model_name='zrodlo',
            name='nazwa_alternatywna',
            field=models.CharField(db_index=True, max_length=1024, null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='zrodlo',
            name='skrot_nazwy_alternatywnej',
            field=models.CharField(db_index=True, max_length=512, null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='autor',
            name='aktualna_jednostka',
            field=models.ForeignKey(on_delete=models.CASCADE, related_name='aktualna_jednostka', blank=True, to='bpp.Jednostka', null=True),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='groups',
            field=models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Group', blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of his/her group.', verbose_name='groups'),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='user_permissions',
            field=models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Permission', blank=True, help_text='Specific permissions for this user.', verbose_name='user permissions'),
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='skrot',
            field=models.CharField(max_length=512, verbose_name=b'Skr\xc3\xb3t', db_index=True),
        )
    ]



import pytest
from dbtemplates.models import Template
from django.db import IntegrityError

from django.contrib.contenttypes.models import ContentType

from bpp.models import Wydawnictwo_Zwarte
from bpp.models.szablondlaopisubibliograficznego import SzablonDlaOpisuBibliograficznego


@pytest.mark.django_db
def test_Template_name_idx():
    Template.objects.create(name="test", content="test")
    with pytest.raises(IntegrityError):
        Template.objects.create(name="test", content="test")


@pytest.mark.django_db
def test_nulltest_idx():
    test_template = Template.objects.create(name="test", content="test")

    if not SzablonDlaOpisuBibliograficznego.objects.filter(model=None).exists():
        # przy ponownym uruchamianiu testow moze byc taka sytuacja
        SzablonDlaOpisuBibliograficznego.objects.create(template=test_template)

    with pytest.raises(IntegrityError):
        SzablonDlaOpisuBibliograficznego.objects.create(template=test_template)


def test_rozne_opisy_rozne_klasy(wydawnictwo_ciagle, wydawnictwo_zwarte):
    SzablonDlaOpisuBibliograficznego.objects.all().delete()

    test_template = Template.objects.create(name="test", content="test")
    second_template = Template.objects.create(name="2nd", content="2nd")

    # Szablon dla każdej klasy
    try:
        sz = SzablonDlaOpisuBibliograficznego.objects.get(model=None)
        sz.template = test_template
        sz.save()
    except SzablonDlaOpisuBibliograficznego.DoesNotExist:
        SzablonDlaOpisuBibliograficznego.objects.create(template=test_template)

    assert wydawnictwo_ciagle.opis_bibliograficzny() == test_template.content
    assert wydawnictwo_zwarte.opis_bibliograficzny() == test_template.content

    # Szablon tylko dla zwartych
    SzablonDlaOpisuBibliograficznego.objects.create(
        model=ContentType.objects.get_for_model(Wydawnictwo_Zwarte),
        template=second_template,
    )

    assert wydawnictwo_ciagle.opis_bibliograficzny() == test_template.content
    assert wydawnictwo_zwarte.opis_bibliograficzny() == second_template.content



# Generated by Django 3.0.11 on 2021-01-03 20:17

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0233_wiecej_doktoratow_dla_jednego"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0234_rekord_mat_nadrzedne")
        ),
        # Odbudowanie tabeli rekord_mat spowoduje skasowanie bpp_uczelnia_ewaluacja_view,
        # stąd potrzeba zaciągnąć ten plik z poprzednich migracji:
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0207_uczelnia_analiza_view")
        ),
    ]



import openpyxl


def find_header_row(sheet, first_column_value, max_row_range=10, max_col_range=10):
    """
    Procedura poszukuje wiersza nagłówkowego.

    :param sheet: :class:`xlrd.sheet.Sheet`
    :param first_column_value: pierwsza poszukiwana wartość
    :type first_column_value: str
    :param max_row_range: maksymalny zakres "w dół" gdzie szukamy,
    :param max_col_range: maksymalny zakres "w prawo" gdzie szukamy.
    :return: zwraca numer wiersza, w którym znajduje się nagłówek.
    """
    for n, row in enumerate(
        sheet.iter_rows(1, max_row=min(max_row_range, sheet.max_row)), start=1
    ):
        for b in range(min(max_col_range, sheet.max_column)):
            f = row[b].value
            if hasattr(f, "upper") and f.upper() == first_column_value.upper():
                return n


def build_mapping(xls_columns, wanted_columns):
    """
    Buduje mapowanie pomiędzy kolumnami dostarczonymi w XLS a kolumnami wymaganymi.

    Przykładowo, przy wejściu:

    xls_columns=["c", "b", "a"]
    wanted_columns={"A": "foo", "B": "bar", "c": "baz"}

    zwraca listę o kolejności elementów:
    ["baz", "bar", "foo"]

    :type xls_columns: list
    :type wanted_columns: dict
    :rtype: list
    """
    ret = []
    upper_wanted_columns = {
        x.upper().strip(): wanted_columns[x] for x in list(wanted_columns.keys())
    }
    for elem in xls_columns:
        if elem.value:
            val = elem.value.upper().strip()
            if val in upper_wanted_columns:
                ret.append(upper_wanted_columns[val])
                continue
        ret.append(None)

    return ret


class ReadDataException(Exception):
    pass


def read_xls_data(
    filename,
    column_mapping,
    header_row_name,
    ignored_sheet_names=None,
    limit=None,
    limit_sheets=None,
    transformations=None,
):
    """
    Importuje dane z pliku XLS - ze wszystkich kolejnych zeszytów.

    :param filename: nazwa pliku XLS do odczytu
    :param column_mapping: mapowanie nazwa kolumny z XLS do atrybutu danych
    :param header_row_name: poszukiwany ciąg znaków z pierwszej kolumny wiersza nagłówka
    :param ignored_sheet_names: nazwy zeszytów ignorowanych - te nie zostaną ujęte w imporcie.
    :param transformations: słownik funkcji, transformujących wartość z arkusza przed zwróceniem jej
    :return:
    """
    book = openpyxl.load_workbook(filename=filename)
    sheets = book.worksheets
    if limit_sheets:
        sheets = sheets[:limit_sheets]
    for sheet in sheets:
        if ignored_sheet_names is not None:
            if sheet.name in ignored_sheet_names:
                continue
        header_row_no = find_header_row(sheet, header_row_name)
        if header_row_no is None:
            raise ReadDataException(
                "Brak wiersza nagłówka, poszukiwano %r" % header_row_name
            )
        header_row_values = list(sheet.iter_rows(header_row_no, header_row_no + 1))[0]
        read_data_mapping = list(
            enumerate(build_mapping(header_row_values, column_mapping))
        )

        max_row = sheet.max_row
        if limit is not None:
            max_row = min(header_row_no + 1 + limit, max_row)

        for row in sheet.iter_rows(header_row_no + 1, max_row):
            dct = {"__sheet__": sheet.title}
            for no, elem in read_data_mapping:
                value = row[no].value

                if transformations is not None:
                    if elem in transformations:
                        value = transformations[elem](value)

                dct[elem] = value
            yield dct



# Generated by Django 2.2.10 on 2020-03-08 21:04

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0201_rekord_mat_z_www"),
    ]

    operations = [
        migrations.AddField(
            model_name="cache_punktacja_dyscypliny",
            name="zapisani_autorzy_z_dyscypliny",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.TextField(), blank=True, null=True, size=None
            ),
        ),
        migrations.AddField(
            model_name="cache_punktacja_dyscypliny",
            name="zapisani_wszyscy_autorzy",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.TextField(), blank=True, null=True, size=None
            ),
        ),
    ]



class AutorSimplePBNAdapter:
    def __init__(self, original):
        self.original = original

    def pbn_get_json(self):
        ret = {"givenNames": self.original.imiona, "lastName": self.original.nazwisko}

        if self.original.pbn_uid_id is not None:
            ret.update({"objectId": self.original.pbn_uid_id})

            s = self.original.pbn_uid
            # Jeżeli powiązany rekord w tabeli pbn_api.Sciencist ma PESEL to
            # dodamy go teraz do eksportowanych informacji:
            pesel = s.value("object", "externalIdentifiers", "PESEL", return_none=True)
            if pesel:
                ret.update({"naturalId": pesel})
        return ret


class AutorZDyscyplinaPBNAdapter(AutorSimplePBNAdapter):
    # To samo, co AutorSimplePBNAdapter, ale eksportuje ORCID

    def pbn_get_json(self):
        ret = super(AutorZDyscyplinaPBNAdapter, self).pbn_get_json()

        # Eksportuj ORCID wyłacznie, gdy nie został wyeksportowany identyfikator PBN
        if (
            self.original.orcid is not None and "objectId" not in ret
        ):  # and self.orcid_w_pbn is True:
            ret.update({"orcidId": self.original.orcid})

        return ret



# -*- coding: utf-8 -*-


from django.db import migrations, models
import integrator2.models.lista_ministerialna


class Migration(migrations.Migration):

    dependencies = [
        ('integrator2', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='listaministerialnaelement',
            options={'ordering': ['nazwa']},
        ),
        migrations.AlterModelOptions(
            name='listaministerialnaintegration',
            options={'ordering': ['-uploaded_on'], 'verbose_name': 'integracja list ministerialnych'},
        ),
        migrations.AlterField(
            model_name='listaministerialnaintegration',
            name='last_updated_on',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='listaministerialnaintegration',
            name='year',
            field=models.IntegerField(default=integrator2.models.lista_ministerialna.last_year, help_text='Rok dla kt\xf3rego zosta\u0142a wydana ta lista ministerialna', verbose_name='Rok'),
        ),
    ]



import logging

from braces.views import GroupRequiredMixin, JSONResponseMixin
from django.views.generic.detail import BaseDetailView

from bpp.const import GR_WPROWADZANIE_DANYCH
from bpp.models import Uczelnia

logger = logging.getLogger("blabla")


class GetWoSAMRInformation(JSONResponseMixin, GroupRequiredMixin, BaseDetailView):
    group_required = GR_WPROWADZANIE_DANYCH
    model = Uczelnia

    def get_context_data(self, **kwargs):
        doi = self.request.POST.get("doi", None)
        pmid = self.request.POST.get("pmid", None)

        if doi is None and pmid is None:
            return {"status": "error", "info": "Podaj DOI lub PubMedID"}

        try:
            res = self.object.wosclient().query_single(pmid, doi)
        except Exception as e:
            logger.exception("Podczas zapytania WOS-AMR")
            return {"status": "error", "info": "%s" % e}

        if res.get("message") == "No Result Found":
            return {"status": "ok", "timesCited": None}

        return {"status": "ok", "timesCited": res.get("timesCited")}

    def post(self, request, *args, **kw):
        self.object = self.get_object()
        return self.render_json_response(self.get_context_data())



# Generated by Django 3.0.7 on 2020-06-07 16:04

from django.db import migrations, models

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0211_oznaczenie_wydania"),
    ]

    operations = [
        migrations.AddField(
            model_name="patent_autor",
            name="upowaznienie_pbn",
            field=models.BooleanField(
                default=False,
                help_text='Tik w polu "upoważnienie PBN" oznacza, że dany autor upoważnił Uczelnię do sprawozdania tej publikacji w ocenie parametrycznej Uczelni',
                verbose_name="Upoważnienie PBN",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_ciagle_autor",
            name="upowaznienie_pbn",
            field=models.BooleanField(
                default=False,
                help_text='Tik w polu "upoważnienie PBN" oznacza, że dany autor upoważnił Uczelnię do sprawozdania tej publikacji w ocenie parametrycznej Uczelni',
                verbose_name="Upoważnienie PBN",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_zwarte_autor",
            name="upowaznienie_pbn",
            field=models.BooleanField(
                default=False,
                help_text='Tik w polu "upoważnienie PBN" oznacza, że dany autor upoważnił Uczelnię do sprawozdania tej publikacji w ocenie parametrycznej Uczelni',
                verbose_name="Upoważnienie PBN",
            ),
        ),
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0212_upowaznienie_pbn")
        ),
        # To musi być wczytane, bo "DROP CASCADE" i gubi widoki ewaluacyjne
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0207_uczelnia_analiza_view")
        ),
    ]



# -*- encoding: utf-8 -*-
import logging

from django.core.management import BaseCommand
from django.db import transaction
from django.db.models import Q

from bpp.models import Wydawnictwo_Zwarte, Wydawca
from bpp.models.struktura import Jednostka, Uczelnia


class Command(BaseCommand):
    help = "Dzieli podwojnych (oddzielonych srednikiem) wydawcow"

    @transaction.atomic
    def handle(self, *args, **options):
        verbosity = int(options["verbosity"])
        logger = logging.getLogger("django")
        if verbosity > 1:
            logger.setLevel(logging.DEBUG)

        for rec in Wydawnictwo_Zwarte.objects.exclude(wydawca=None).filter(
            wydawca__nazwa__contains=";"
        ):
            nazwa_wydawcy = rec.wydawca.nazwa

            z_poziomami = []
            bez_poziomow = []

            logger.info("")
            logger.info(f"Rekord: {rec}")
            logger.info(f". ma takiego wydawcę: {nazwa_wydawcy}")

            for elem in rec.wydawca.nazwa.split(";"):
                elem = elem.strip()
                try:
                    nw = Wydawca.objects.get(nazwa=elem)
                    ma_poziomy = nw.poziom_wydawcy_set.exists()
                    logger.info(f"+ człon {elem} -> {nw}, ma_poziomy: {ma_poziomy}...")
                except Wydawca.DoesNotExist:
                    logger.info(f"- człon {elem} nie pasuje nigdzie...")
                    continue

                if ma_poziomy:
                    z_poziomami.append(nw)
                else:
                    bez_poziomow.append(nw)

            for tablica in [z_poziomami, bez_poziomow]:
                if not tablica:
                    continue

                for wydawca in tablica:
                    rec.wydawca = wydawca
                    rec.wydawca_opis = nazwa_wydawcy.replace(rec.wydawca.nazwa, "")
                    rec.save()
                    logger.info(
                        f"-> zmienie go na {wydawca.nazwa}, reszta {rec.wydawca_opis}"
                    )
                    break
                break



from unittest.mock import Mock

import pytest
from model_bakery import baker

from integrator2.models.lista_ministerialna import ListaMinisterialnaElement

from bpp.models.zrodlo import Punktacja_Zrodla, Zrodlo


def test_models_lista_ministerialna_input_file_to_dict_stream(lmi):
    gen = lmi.input_file_to_dict_stream()
    next(gen)
    res = next(gen)
    assert res["nazwa"] == "AAPG BULLETIN"


def test_models_lista_ministerialna_b_alt(lmi_b):
    gen = lmi_b.input_file_to_dict_stream()
    next(gen)
    res = next(gen)
    assert res["nazwa"] == "„Studia Etnologiczne i Antropologiczne”"


def test_models_lista_ministerialna_b(lmi_b):
    gen = lmi_b.input_file_to_dict_stream()
    next(gen)
    res = next(gen)
    assert res["nazwa"] == "„Studia Etnologiczne i Antropologiczne”"


def test_models_lista_ministerialna_c(lmi_c):
    gen = lmi_c.input_file_to_dict_stream()
    next(gen)
    res = next(gen)
    assert res["nazwa"] == "19th Century music"


@pytest.mark.django_db
def test_models_lista_ministerialna_dict_stream_to_db(lmi):
    lmi.dict_stream_to_db(limit=20)
    assert ListaMinisterialnaElement.objects.all().count() == 20


@pytest.mark.django_db
def test_models_lista_ministerialna_match_single_record(lmi):
    z = baker.make(Zrodlo, nazwa="AAPG BULLETIN", issn="1111-1111", e_issn="1234-1234")

    elem = Mock(issn="1111-1111", e_issn=None, nazwa=None)
    lmi.match_single_record(elem)
    assert elem.zrodlo == z

    elem = Mock(issn=None, e_issn="1234-1234", nazwa=None)
    lmi.match_single_record(elem)
    assert elem.zrodlo == z

    elem = Mock(issn=None, e_issn=None, nazwa="AAPG BULLETIN")
    lmi.match_single_record(elem)
    assert elem.zrodlo == z


@pytest.mark.django_db
def test_models_integrate_single_record(lmi):
    z = baker.make(Zrodlo, nazwa="AAPG BULLETIN", issn="1111-1111", e_issn="1234-1234")
    elem = Mock(issn="1111-1111", e_issn=None, nazwa=None, zrodlo=z, punkty_kbn=999)
    elem.parent = Mock(year=2005)

    lmi.integrate_single_record(elem)
    pz = Punktacja_Zrodla.objects.get(zrodlo=z)
    assert pz.punkty_kbn == 999



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-29 16:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0144_wyrzuc_w_z_informacji'),
    ]

    operations = [
        migrations.AddField(
            model_name='uczelnia',
            name='wyszukiwanie_rekordy_na_strone_anonim',
            field=models.SmallIntegerField(default=200, help_text='Ilość rekordów w wyszukiwaniu powyżej której znika opcja"Pokaż wszystkie" i "Drukuj" dla użytkownika anonimowego. Nie jest zalecane ustawianie powyżej 500. ', verbose_name='Ilość rekordów na stronę - anonim'),
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='wyszukiwanie_rekordy_na_strone_zalogowany',
            field=models.SmallIntegerField(default=5000, help_text='Ilość rekordów w wyszukiwaniu powyżej której znika opcja"Pokaż wszystkie" i "Drukuj" dla użytkownika zalogowanego. Nie jest zalecane ustawianie powyżej 5000. ', verbose_name='Ilość rekordów na stronę - anonim'),
        ),
    ]



# -*- encoding: utf-8 -*-


import json

from bpp.views.api.strona_tom_nr_zeszytu import StronaTomNrZeszytuView


def test_api_strona_tom_nr_zeszytu():
    class FakeReq2:
        POST = {'s': 's. 22-33',
                'i': '1992 vol. 5 z. 4'}

    x = StronaTomNrZeszytuView().post(FakeReq2())
    ret = json.loads(x.content)

    assert ret['strony'] == '22-33'
    assert ret['tom'] == '5'
    assert ret['nr_zeszytu'] == '4'



import pytest

from bpp.models import Wydawnictwo_Ciagle
from long_running.util import wait_for_object


def test_wait_for_object(wydawnictwo_ciagle):
    with pytest.warns(DeprecationWarning):
        assert (
            wait_for_object(Wydawnictwo_Ciagle, wydawnictwo_ciagle.pk)
            == wydawnictwo_ciagle
        )



# Generated by Django 3.0.11 on 2021-02-01 12:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0234_rekord_mat_nadrzedne"),
    ]

    operations = [
        migrations.AddField(
            model_name="ukryj_status_korekty",
            name="podglad",
            field=models.BooleanField(
                default=True,
                help_text="Uniemożliwia podgląd prac na stronie szczegółów rekordu oraz uniemożliwia wyszukanie takich prac przez pole globalnego wyszukania.",
            ),
        ),
    ]



import re
import warnings

from denorm import denormalized, depend_on_related
from dirtyfields.dirtyfields import DirtyFieldsMixin
from django.db import models
from django.db.models import CASCADE, PROTECT, JSONField
from django.db.models.expressions import RawSQL

from django.contrib.contenttypes.fields import GenericRelation
from django.contrib.postgres.fields import ArrayField

from bpp.models import (
    BazaModeluStreszczen,
    DodajAutoraMixin,
    MaProcentyMixin,
    ModelOpcjonalnieNieEksportowanyDoAPI,
    ModelZMiejscemPrzechowywania,
    ModelZOplataZaPublikacje,
    ModelZPBN_UID,
)
from bpp.models.abstract import (
    BazaModeluOdpowiedzialnosciAutorow,
    DwaTytuly,
    ModelPunktowany,
    ModelRecenzowany,
    ModelTypowany,
    ModelWybitny,
    ModelZAbsolutnymUrl,
    ModelZAdnotacjami,
    ModelZCharakterem,
    ModelZDOI,
    ModelZeStatusem,
    ModelZeSzczegolami,
    ModelZeZnakamiWydawniczymi,
    ModelZInformacjaZ,
    ModelZISBN,
    ModelZISSN,
    ModelZKonferencja,
    ModelZLiczbaCytowan,
    ModelZOpenAccess,
    ModelZPrzeliczaniemDyscyplin,
    ModelZPubmedID,
    ModelZRokiem,
    ModelZSeria_Wydawnicza,
    ModelZWWW,
    Wydawnictwo_Baza,
)
from bpp.models.autor import Autor
from bpp.models.nagroda import Nagroda
from bpp.models.system import Zewnetrzna_Baza_Danych
from bpp.models.util import ZapobiegajNiewlasciwymCharakterom
from bpp.models.wydawca import Wydawca


class Wydawnictwo_Zwarte_Autor(
    DirtyFieldsMixin,
    BazaModeluOdpowiedzialnosciAutorow,
):
    """Model zawierający informację o przywiązaniu autorów do wydawnictwa
    zwartego."""

    rekord = models.ForeignKey(
        "Wydawnictwo_Zwarte", CASCADE, related_name="autorzy_set"
    )

    class Meta:
        verbose_name = "powiązanie autora z wyd. zwartym"
        verbose_name_plural = "powiązania autorów z wyd. zwartymi"
        app_label = "bpp"
        ordering = ("kolejnosc",)
        unique_together = [
            ("rekord", "autor", "typ_odpowiedzialnosci"),
            # Tu musi być autor, inaczej admin nie pozwoli wyedytować
            ("rekord", "autor", "kolejnosc"),
        ]


MIEJSCE_I_ROK_MAX_LENGTH = 256


class Wydawnictwo_Zwarte_Baza(
    Wydawnictwo_Baza,
    DwaTytuly,
    ModelZRokiem,
    ModelZeStatusem,
    ModelZWWW,
    ModelZPubmedID,
    ModelZDOI,
    ModelRecenzowany,
    ModelPunktowany,
    ModelTypowany,
    ModelZeSzczegolami,
    ModelZInformacjaZ,
    ModelZISBN,
    ModelZAdnotacjami,
    ModelZAbsolutnymUrl,
    ModelZLiczbaCytowan,
    ModelZMiejscemPrzechowywania,
    ModelOpcjonalnieNieEksportowanyDoAPI,
):
    """Baza dla klas Wydawnictwo_Zwarte oraz Praca_Doktorska_Lub_Habilitacyjna"""

    miejsce_i_rok = models.CharField(
        max_length=MIEJSCE_I_ROK_MAX_LENGTH,
        blank=True,
        null=True,
        help_text="""Przykładowo:
        Warszawa 2012. Wpisz proszę najpierw miejsce potem rok; oddziel
        spacją.""",
    )

    wydawca = models.ForeignKey(Wydawca, PROTECT, null=True, blank=True)
    wydawca_opis = models.CharField(
        "Wydawca - szczegóły", max_length=256, null=True, blank=True
    )

    oznaczenie_wydania = models.CharField(max_length=400, null=True, blank=True)

    def get_wydawnictwo(self):
        # Zwróć nazwę wydawcy + pole wydawca_opis lub samo pole wydawca_opis, jeżeli
        # wydawca (indeksowany) nie jest ustalony
        if self.wydawca_id is None:
            return self.wydawca_opis

        opis = self.wydawca_opis or ""
        try:
            if opis[0] in ".;-/,":
                # Nie wstawiaj spacji między wydawcę a opis jeżeli zaczyna się od kropki, przecinka itp
                return f"{self.wydawca.nazwa}{opis}".strip()
        except IndexError:
            pass

        return f"{self.wydawca.nazwa} {opis}".strip()

    def set_wydawnictwo(self, value):
        warnings.warn("W przyszlosci uzyj 'wydawca_opis'", DeprecationWarning)
        self.wydawca_opis = value

    wydawnictwo = property(get_wydawnictwo, set_wydawnictwo)

    redakcja = models.TextField(null=True, blank=True)

    class Meta:
        abstract = True


class ModelZOpenAccessWydawnictwoZwarte(ModelZOpenAccess):
    openaccess_tryb_dostepu = models.ForeignKey(
        "Tryb_OpenAccess_Wydawnictwo_Zwarte", CASCADE, blank=True, null=True
    )

    class Meta:
        abstract = True


rok_regex = re.compile(r"\s[12]\d\d\d")


class Wydawnictwo_Zwarte_Manager(models.Manager):
    def wydawnictwa_nadrzedne_dla_innych(self):
        return (
            self.exclude(wydawnictwo_nadrzedne_id=None)
            .values_list("wydawnictwo_nadrzedne_id", flat=True)
            .distinct()
        )


class Wydawnictwo_Zwarte(
    ZapobiegajNiewlasciwymCharakterom,
    Wydawnictwo_Zwarte_Baza,
    ModelZCharakterem,
    ModelZOpenAccessWydawnictwoZwarte,
    ModelZeZnakamiWydawniczymi,
    ModelZKonferencja,
    ModelZSeria_Wydawnicza,
    ModelZISSN,
    ModelWybitny,
    ModelZPBN_UID,
    ModelZOplataZaPublikacje,
    MaProcentyMixin,
    DodajAutoraMixin,
    DirtyFieldsMixin,
    ModelZPrzeliczaniemDyscyplin,
):
    """Wydawnictwo zwarte, czyli: książki, broszury, skrypty, fragmenty,
    doniesienia zjazdowe."""

    objects = Wydawnictwo_Zwarte_Manager()

    autor_rekordu_klass = Wydawnictwo_Zwarte_Autor
    autorzy = models.ManyToManyField(Autor, through=autor_rekordu_klass)

    wydawnictwo_nadrzedne = models.ForeignKey(
        "self",
        CASCADE,
        blank=True,
        null=True,
        help_text="""Jeżeli dodajesz rozdział,
        tu wybierz pracę, w ramach której dany rozdział występuje.""",
        related_name="wydawnictwa_powiazane_set",
    )

    calkowita_liczba_autorow = models.PositiveIntegerField(
        blank=True,
        null=True,
        help_text="""Jeżeli dodajesz monografię, wpisz
        tutaj całkowitą liczbę autorów monografii. Ta informacja zostanie
        użyta w eksporcie danych do PBN. Jeżeli informacja ta nie zostanie
        uzupełiona, wartość tego pola zostanie obliczona i będzie to ilość
        wszystkich autorów przypisanych do danej monografii""",
    )

    calkowita_liczba_redaktorow = models.PositiveIntegerField(
        blank=True,
        null=True,
        help_text="""Jeżeli dodajesz monografię, wpisz tutaj całkowitą liczbę
        redaktorów monografii. Ta informacja zostanie użyta w eksporcie
        danych do PBN. Jeżeli pole to nie zostanie uzupełnione, wartość ta
        zostanie obliczona i będzie to ilość wszystkich redaktorów
        przypisanych do danej monografii""",
    )

    nagrody = GenericRelation(Nagroda)

    class Meta:
        verbose_name = "wydawnictwo zwarte"
        verbose_name_plural = "wydawnictwa zwarte"
        app_label = "bpp"

    def wydawnictwa_powiazane_posortowane(self):
        """
        Sortowanie wydawnictw powiązanych wg pierwszej liczby dziesiętnej występującej w polu 'Strony'
        """
        return self.wydawnictwa_powiazane_set.order_by(
            RawSQL(
                r"CAST((regexp_match(COALESCE(bpp_wydawnictwo_zwarte.strony, '99999999'), '(\d+)'))[1] AS INT)",
                "",
            )
        )

    #
    # Cache framework by django-denorm-iplweb
    #

    denorm_always_skip = ("ostatnio_zmieniony",)

    @denormalized(JSONField, blank=True, null=True)
    @depend_on_related(
        "bpp.Wydawnictwo_Zwarte_Autor",
        only=(
            "autor_id",
            "jednostka_id",
            "typ_odpowiedzialnosci_id",
            "afiliuje",
            "dyscyplina_naukowa_id",
            "upowaznienie_pbn",
            "przypieta",
        ),
    )
    @depend_on_related("bpp.Wydawca", only=("lista_poziomow", "alias_dla_id"))
    def cached_punkty_dyscyplin(self):
        # TODO: idealnie byłoby uzależnić zmiane od pola 'rok' które by było identyczne
        # dla bpp.Poziom_Wydawcy, rok i id z nadrzędnego. Składnia SQLowa ewentualnie
        # jakis zapis django-podobny mile widziany.
        return self.przelicz_punkty_dyscyplin()

    @denormalized(models.TextField, default="")
    @depend_on_related("self", "wydawnictwo_nadrzedne")
    @depend_on_related(
        "bpp.Wydawnictwo_Zwarte_Autor",
        only=("zapisany_jako", "typ_odpowiedzialnosci_id", "kolejnosc"),
    )
    @depend_on_related("bpp.Wydawca", only=("nazwa", "alias_dla_id"))
    @depend_on_related("bpp.Charakter_Formalny")
    @depend_on_related("bpp.Typ_KBN")
    @depend_on_related("bpp.Status_Korekty")
    def opis_bibliograficzny_cache(self):
        return self.opis_bibliograficzny()

    @denormalized(ArrayField, base_field=models.TextField(), blank=True, null=True)
    @depend_on_related(
        "bpp.Autor",
        only=(
            "nazwisko",
            "imiona",
        ),
    )
    @depend_on_related("bpp.Wydawnictwo_Zwarte_Autor", only=("kolejnosc",))
    def opis_bibliograficzny_autorzy_cache(self):
        return [
            f"{x.autor.nazwisko} {x.autor.imiona}" for x in self.autorzy_dla_opisu()
        ]

    @denormalized(models.TextField, blank=True, null=True)
    @depend_on_related(
        "bpp.Wydawnictwo_Zwarte_Autor",
        only=("zapisany_jako", "kolejnosc"),
    )
    def opis_bibliograficzny_zapisani_autorzy_cache(self):
        return ", ".join([x.zapisany_jako for x in self.autorzy_dla_opisu()])

    @denormalized(
        models.SlugField,
        max_length=400,
        unique=True,
        db_index=True,
        null=True,
        blank=True,
    )
    @depend_on_related(
        "bpp.Wydawnictwo_Zwarte_Autor",
        only=("zapisany_jako", "kolejnosc"),
    )
    @depend_on_related(
        "bpp.Autor",
        only=("nazwisko", "imiona"),
    )
    @depend_on_related("self", "wydawnictwo_nadrzedne")
    def slug(self):
        return self.get_slug()

    def clean(self):
        DwaTytuly.clean(self)
        ModelZOplataZaPublikacje.clean(self)


class Wydawnictwo_Zwarte_Zewnetrzna_Baza_Danych(models.Model):
    rekord = models.ForeignKey(
        Wydawnictwo_Zwarte, CASCADE, related_name="zewnetrzna_baza_danych"
    )
    baza = models.ForeignKey(Zewnetrzna_Baza_Danych, CASCADE)
    info = models.CharField(
        verbose_name="Informacje dodatkowe", max_length=512, blank=True, null=True
    )

    class Meta:
        verbose_name = "powiązanie wyd. zwartego z zewn. bazami danych"
        verbose_name_plural = "powiązania wyd. zwartych z zewn. bazami danych"


class Wydawnictwo_Zwarte_Streszczenie(BazaModeluStreszczen):
    rekord = models.ForeignKey(Wydawnictwo_Zwarte, CASCADE, related_name="streszczenia")

    class Meta:
        verbose_name = "streszczenie wydawnictwa zwartego"
        verbose_name_plural = "streszczenia wydawnictw zwatrtych"

    def __str__(self):
        if self.jezyk_streszczenia_id is not None:
            return (
                f"Streszczenie rekordu {self.rekord} w języku {self.jezyk_streszczenia}"
            )
        return f"Streszczenie rekordu {self.rekord}"



import json

from braces.views import GroupRequiredMixin, LoginRequiredMixin
from dal import autocomplete
from dal_select2_queryset_sequence.views import Select2QuerySetSequenceView
from django import http
from django.core.exceptions import ImproperlyConfigured
from django.db.models.aggregates import Count
from django.db.models.query_utils import Q
from queryset_sequence import QuerySetSequence
from taggit.models import Tag

from import_common.core import normalized_db_isbn
from import_common.normalization import normalize_isbn

from django.contrib.auth.mixins import UserPassesTestMixin
from django.contrib.postgres.search import TrigramSimilarity

from bpp import const
from bpp.const import CHARAKTER_OGOLNY_KSIAZKA, GR_WPROWADZANIE_DANYCH
from bpp.jezyk_polski import warianty_zapisanego_nazwiska
from bpp.lookups import SearchQueryStartsWith
from bpp.models import (
    Autor_Dyscyplina,
    Dyscyplina_Naukowa,
    Jednostka,
    Uczelnia,
    Wydawca,
    Zewnetrzna_Baza_Danych,
)
from bpp.models.autor import Autor
from bpp.models.cache import Rekord
from bpp.models.konferencja import Konferencja
from bpp.models.nagroda import OrganPrzyznajacyNagrody
from bpp.models.patent import Patent, Patent_Autor
from bpp.models.praca_doktorska import Praca_Doktorska
from bpp.models.praca_habilitacyjna import Praca_Habilitacyjna
from bpp.models.profile import BppUser
from bpp.models.seria_wydawnicza import Seria_Wydawnicza
from bpp.models.struktura import Wydzial
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle, Wydawnictwo_Ciagle_Autor
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte, Wydawnictwo_Zwarte_Autor
from bpp.models.zrodlo import Rodzaj_Zrodla, Zrodlo
from bpp.util import fulltext_tokenize


class PublicTaggitTagAutocomplete(autocomplete.Select2QuerySetView):
    create_field = None

    def get_queryset(self):
        qs = Tag.objects.all()
        if self.q:
            qs = qs.filter(name__icontains=self.q)
        return qs


class Wydawnictwo_NadrzedneAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):

        qs = Wydawnictwo_Zwarte.objects.filter(
            charakter_formalny__charakter_ogolny=CHARAKTER_OGOLNY_KSIAZKA
        )

        if self.q:
            qs = qs.filter(tytul_oryginalny__icontains=self.q)
        return qs


class Wydawnictwo_CiagleAdminAutocomplete(
    LoginRequiredMixin, autocomplete.Select2QuerySetView
):
    def get_queryset(self):
        qs = Wydawnictwo_Ciagle.objects.all()
        if self.q:
            qs = qs.filter(tytul_oryginalny__icontains=self.q)
        return qs


class Wydawnictwo_ZwarteAdminAutocomplete(
    LoginRequiredMixin, autocomplete.Select2QuerySetView
):
    def get_queryset(self):
        qs = Wydawnictwo_Zwarte.objects.all()
        if self.q:
            qs = qs.filter(tytul_oryginalny__icontains=self.q)
        return qs


class PublicWydawnictwo_NadrzedneAutocomplete(Wydawnictwo_NadrzedneAutocomplete):
    create_field = None

    def get_queryset(self):
        """
        :test: :py:class:`bpp.tests.test_autocomplete`
        """

        # Publiczna wyszukiwarka dla wydawnictw nadrzędnych powinna wyszukiwać wyłącznie rekordy,
        # które są już wydawnictwami nadrzędnymi dla jakichś rekordów:

        qs = Wydawnictwo_Zwarte.objects.filter(
            pk__in=Wydawnictwo_Zwarte.objects.exclude(wydawnictwo_nadrzedne_id=None)
            .values_list("wydawnictwo_nadrzedne_id")
            .distinct()
        )

        if self.q:
            qs = qs.filter(tytul_oryginalny__icontains=self.q)
        return qs


class JednostkaMixin:
    def get_result_label(self, result):
        if result is not None:
            if hasattr(result, "wydzial") and result.wydzial is not None:
                return f"{result.nazwa} ({result.wydzial.skrot})"
            return f"{result.nazwa} (bez wydziału)"


class JednostkaAutocomplete(JednostkaMixin, autocomplete.Select2QuerySetView):
    qset = Jednostka.objects.all().select_related("wydzial")

    def get_queryset(self):
        qs = self.qset
        if self.q:
            qs = qs.filter(Q(nazwa__icontains=self.q) | Q(skrot__icontains=self.q))
        return qs.order_by(*Jednostka.objects.get_default_ordering())


class LataAutocomplete(autocomplete.Select2QuerySetView):
    qset = (
        Rekord.objects.all().values_list("rok", flat=True).distinct().order_by("-rok")
    )

    def get_queryset(self):
        qs = self.qset
        if self.q:
            qs = qs.filter(rok=self.q)
        return qs

    def get_result_value(self, result):
        return result

    def get_result_label(self, result):
        return str(result)


class NazwaMixin:
    def get_queryset(self):
        qs = self.qset
        if self.q:
            self.q = self.q.strip()
            qs = qs.filter(nazwa__icontains=self.q)
        return qs


class NazwaTrigramMixin:
    MIN_TRIGRAM_MATCH = 0.05

    def get_queryset(self):
        qs = self.qset
        if self.q:

            self.q = self.q.strip()
            qs = (
                qs.annotate(similarity=TrigramSimilarity("nazwa", self.q))
                .filter(similarity__gte=self.MIN_TRIGRAM_MATCH)
                .order_by("-similarity")[:10]
            )
        return qs


class NazwaLubSkrotMixin:
    def get_queryset(self):
        qs = self.qset
        if self.q:
            qs = qs.filter(Q(nazwa__icontains=self.q) | Q(skrot__icontains=self.q))
        return qs


class KonferencjaAutocomplete(
    NazwaMixin, LoginRequiredMixin, autocomplete.Select2QuerySetView
):
    create_field = "nazwa"
    qset = Konferencja.objects.all()

    def get_result_label(self, result):
        return f"{Konferencja.TK_SYMBOLE[result.typ_konferencji]} {str(result)}"

    def create_object(self, text):
        return self.get_queryset().create(nazwa=text.strip())


class WydawcaAutocomplete(
    NazwaTrigramMixin, LoginRequiredMixin, autocomplete.Select2QuerySetView
):
    create_field = "nazwa"
    qset = Wydawca.objects.all()

    def create_object(self, text):
        return self.get_queryset().create(nazwa=text.strip())


class PublicKonferencjaAutocomplete(NazwaMixin, autocomplete.Select2QuerySetView):
    qset = Konferencja.objects.all()


class Seria_WydawniczaAutocomplete(
    NazwaMixin, LoginRequiredMixin, autocomplete.Select2QuerySetView
):
    create_field = "nazwa"
    qset = Seria_Wydawnicza.objects.all()


class WydzialAutocomplete(NazwaLubSkrotMixin, autocomplete.Select2QuerySetView):
    qset = Wydzial.objects.all()


class PublicWydzialAutocomplete(NazwaLubSkrotMixin, autocomplete.Select2QuerySetView):
    qset = Wydzial.objects.filter(widoczny=True)


class OrganPrzyznajacyNagrodyAutocomplete(NazwaMixin, autocomplete.Select2QuerySetView):
    qset = OrganPrzyznajacyNagrody.objects.all()


class WidocznaJednostkaAutocomplete(JednostkaAutocomplete):
    qset = Jednostka.objects.widoczne().select_related("wydzial")


class PublicJednostkaAutocomplete(JednostkaAutocomplete):
    qset = Jednostka.objects.publiczne().select_related("wydzial")


def autocomplete_create_error(msg):
    class Error:
        pk = -1

        def __str__(self):
            return msg

    return Error()


class PublicZrodloAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = Zrodlo.objects.all()
        if self.q:
            for token in [x.strip() for x in self.q.split(" ") if x.strip()]:
                qs = qs.filter(
                    Q(nazwa__icontains=token)
                    | Q(poprzednia_nazwa__icontains=token)
                    | Q(nazwa_alternatywna__icontains=token)
                    | Q(skrot__istartswith=token)
                    | Q(skrot_nazwy_alternatywnej__istartswith=token)
                )
        return qs


class ZrodloAutocomplete(GroupRequiredMixin, PublicZrodloAutocomplete):
    create_field = "nazwa"
    group_required = GR_WPROWADZANIE_DANYCH

    def create_object(self, text):
        try:
            rz = Rodzaj_Zrodla.objects.get(nazwa="periodyk")
        except Rodzaj_Zrodla.DoesNotExist:
            return autocomplete_create_error(
                "Nie można utworzyć źródła - brak zdefiniowanego"
                " rodzaju źródła 'periodyk'"
            )

        return self.get_queryset().create(nazwa=text.strip(), rodzaj=rz)


class AutorAutocompleteBase(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = Autor.objects.all()
        if self.q:
            tokens = fulltext_tokenize(self.q)
            query = SearchQueryStartsWith(
                "&".join([token + ":*" for token in tokens if token]),
                config="bpp_nazwy_wlasne",
            )

            qs = qs.filter(search=query)

            qs = (
                qs.annotate(Count("wydawnictwo_ciagle"))
                .select_related("tytul")
                .order_by("-wydawnictwo_ciagle__count")
            )

        return qs


class AutorAutocomplete(GroupRequiredMixin, AutorAutocompleteBase):
    create_field = "nonzero"
    group_required = GR_WPROWADZANIE_DANYCH

    err = autocomplete_create_error(
        "Wpisz nazwisko, potem imiona. Wyrazy oddziel spacją. "
    )

    def create_object(self, text):
        try:
            return Autor.objects.create_from_string(text)
        except ValueError:
            return self.err


class PublicAutorAutocomplete(AutorAutocompleteBase):
    pass


class AutorZUczelniAutocopmlete(AutorAutocomplete):
    pass


class StaffRequired(UserPassesTestMixin):
    def test_func(self):
        return self.request.user.is_staff


def jest_czyms(s, dlugosc):
    if s is not None:
        if len(s) == dlugosc and s.find(" ") == -1:
            return True
    return False


def jest_orcid(s):
    return jest_czyms(s, const.ORCID_LEN)


def jest_pbn_uid(s):
    return jest_czyms(s, const.PBN_UID_LEN)


def globalne_wyszukiwanie_autora(querysets, q):
    def _fun(qry):
        return (
            qry.annotate(Count("wydawnictwo_ciagle"))
            .only(
                "pk",
                "nazwisko",
                "imiona",
                "poprzednie_nazwiska",
                "tytul__skrot",
                "pseudonim",
            )
            .select_related("tytul")
            .order_by("-wydawnictwo_ciagle__count")
        )

    if jest_orcid(q):
        querysets.append(_fun(Autor.objects.filter(orcid__icontains=q)))

    if jest_pbn_uid(q):
        querysets.append(_fun(Autor.objects.filter(pbn_uid_id=q)))

    querysets.append(_fun(Autor.objects.fulltext_filter(q)))


def globalne_wyszukiwanie_jednostki(querysets, s):
    def _fun(qry):
        return qry.only("pk", "nazwa", "wydzial__skrot").select_related("wydzial")

    querysets.append(_fun(Jednostka.objects.fulltext_filter(s)))

    if jest_pbn_uid(s):
        querysets.append(_fun(Jednostka.objects.filter(pbn_uid_id=s)))


def globalne_wyszukiwanie_zrodla(querysets, s):
    def _fun(qry):
        return qry.only("pk", "nazwa", "poprzednia_nazwa")

    querysets.append(_fun(Zrodlo.objects.fulltext_filter(s)))

    if jest_pbn_uid(s):
        querysets.append(_fun(Zrodlo.objects.filter(pbn_uid_id=s)))


class GlobalNavigationAutocomplete(Select2QuerySetSequenceView):
    paginate_by = 20

    def get_queryset(self):
        if not hasattr(self, "q"):
            return []

        if not self.q:
            return []

        querysets = []
        globalne_wyszukiwanie_jednostki(querysets, self.q)

        globalne_wyszukiwanie_autora(querysets, self.q)

        globalne_wyszukiwanie_zrodla(querysets, self.q)

        # Rekord

        rekord_qset_ftx = Rekord.objects.fulltext_filter(self.q)

        rekord_qset_doi = Rekord.objects.filter(doi__iexact=self.q)
        rekord_qset_isbn = Rekord.objects.filter(isbn__iexact=self.q)
        rekord_qset_pbn = None
        if jest_pbn_uid(self.q):
            rekord_qset_pbn = Rekord.objects.filter(pbn_uid_id=self.q)

        qry = Q(pk__in=rekord_qset_doi.values_list("pk"))
        qry |= Q(pk__in=rekord_qset_isbn)
        if rekord_qset_pbn:
            qry |= Q(pk__in=rekord_qset_pbn.values_list("pk"))

        rekord_qset = Rekord.objects.filter(qry).only("tytul_oryginalny")

        if hasattr(self, "request") and self.request.user.is_anonymous:
            uczelnia = Uczelnia.objects.get_for_request(self.request)
            if uczelnia is not None:
                rekord_qset_ftx = rekord_qset_ftx.exclude(
                    status_korekty_id__in=uczelnia.ukryte_statusy("podglad")
                )

                rekord_qset = rekord_qset.exclude(
                    status_korekty_id__in=uczelnia.ukryte_statusy("podglad")
                )
        querysets.append(rekord_qset_ftx)
        querysets.append(rekord_qset)

        this_is_an_id = False
        try:
            this_is_an_id = int(self.q)
        except (TypeError, ValueError):
            pass

        if this_is_an_id:
            querysets.append(
                Rekord.objects.extra(where=["id[2]=%s" % this_is_an_id]).only(
                    "tytul_oryginalny"
                )
            )

        ret = QuerySetSequence(*querysets)
        return self.mixup_querysets(ret)


class AdminNavigationAutocomplete(StaffRequired, Select2QuerySetSequenceView):
    paginate_by = 30

    def get_queryset(self):
        if not self.q:
            return []

        if len(self.q) < 1:
            return []

        querysets = []

        querysets.append(
            BppUser.objects.filter(username__icontains=self.q).only("pk", "username")
        )

        globalne_wyszukiwanie_jednostki(querysets, self.q)

        querysets.append(
            Konferencja.objects.filter(
                Q(nazwa__icontains=self.q) | Q(skrocona_nazwa__icontains=self.q)
            ).only("pk", "nazwa", "baza_inna", "baza_wos", "baza_scopus")
        )

        globalne_wyszukiwanie_autora(querysets, self.q)

        globalne_wyszukiwanie_zrodla(querysets, self.q)

        for klass in [
            Wydawnictwo_Zwarte,
            Wydawnictwo_Ciagle,
            Patent,
            Praca_Doktorska,
            Praca_Habilitacyjna,
        ]:

            filter = Q(tytul_oryginalny__icontains=self.q)

            try:
                int(self.q)
                filter |= Q(pk=self.q)
            except (TypeError, ValueError):
                pass

            if klass != Patent:
                filter |= Q(doi__iexact=self.q)

            if len(self.q) == 24 and self.q.find(" ") == -1:
                if "pbn_uid" in [fld.name for fld in klass._meta.fields]:
                    filter |= Q(pbn_uid__pk=self.q)

            annotate_isbn = False
            if hasattr(klass, "isbn"):
                ni = normalize_isbn(self.q)
                if len(ni) < 20:
                    filter |= Q(normalized_isbn=ni)
                    annotate_isbn = True

            if annotate_isbn:
                qset = klass.objects.annotate(
                    normalized_isbn=normalized_db_isbn
                ).filter(filter)
            else:
                qset = klass.objects.filter(filter)

            querysets.append(qset.only("tytul_oryginalny"))

        ret = QuerySetSequence(*querysets)
        return self.mixup_querysets(ret)


class ZapisanyJakoAutocomplete(autocomplete.Select2ListView):
    def get(self, request, *args, **kwargs):
        # Celem spatchowania tej funkcji jest zmiana tekstu 'Create "%s"'
        # na po prostu '%s'. Poza tym jest to kalka z autocomplete.Select2ListView
        results = self.get_list()
        create_option = []
        if self.q:
            results = [x for x in results if self.q.lower() in x.lower()]
            if hasattr(self, "create"):
                create_option = [{"id": self.q, "text": self.q, "create_id": True}]
        return http.HttpResponse(
            json.dumps(
                {"results": [dict(id=x, text=x) for x in results] + create_option}
            ),
            content_type="application/json",
        )

    def post(self, request):
        # Hotfix dla django-autocomplete-light w wersji 3.3.0-rc5, pull
        # request dla problemu zgłoszony tutaj:
        # https://github.com/yourlabs/django-autocomplete-light/issues/977
        if not hasattr(self, "create"):
            raise ImproperlyConfigured('Missing "create()"')

        text = request.POST.get("text", None)

        if text is None:
            return http.HttpResponseBadRequest()

        text = self.create(text)

        if text is None:
            return http.HttpResponseBadRequest()

        return http.JsonResponse(
            {
                "id": text,
                "text": text,
            }
        )

    def create(self, text):
        return text

    def get_list(self):
        autor = self.forwarded.get("autor", None)

        if autor is None:
            return ["(... może najpierw wybierz autora)"]

        try:
            autor_id = int(autor)
            a = Autor.objects.get(pk=autor_id)
        except (KeyError, ValueError):
            return [
                'Błąd. Wpisz poprawne dane w pole "Autor".',
            ]
        except Autor.DoesNotExist:
            return [
                'Błąd. Wpisz poprawne dane w pole "Autor".',
            ]
        return list(
            set(
                list(
                    warianty_zapisanego_nazwiska(
                        a.imiona, a.nazwisko, a.poprzednie_nazwiska
                    )
                )
            )
        )


class PodrzednaPublikacjaHabilitacyjnaAutocomplete(Select2QuerySetSequenceView):
    def get_queryset(self):
        wydawnictwa_zwarte = Wydawnictwo_Zwarte.objects.all()
        wydawnictwa_ciagle = Wydawnictwo_Ciagle.objects.all()
        patenty = Patent.objects.all()

        qs = QuerySetSequence(wydawnictwa_ciagle, wydawnictwa_zwarte, patenty)

        autor_id = self.forwarded.get("autor", None)
        if autor_id is None:
            return qs.none()

        try:
            autor = Autor.objects.get(pk=int(autor_id))
        except (TypeError, ValueError, Autor.DoesNotExist):
            return qs.none()

        wydawnictwa_zwarte = Wydawnictwo_Zwarte.objects.filter(
            pk__in=Wydawnictwo_Zwarte_Autor.objects.filter(autor=autor).only("rekord")
        )
        wydawnictwa_ciagle = Wydawnictwo_Ciagle.objects.filter(
            pk__in=Wydawnictwo_Ciagle_Autor.objects.filter(autor=autor).only("rekord")
        )

        patenty = Patent.objects.filter(
            pk__in=Patent_Autor.objects.filter(autor=autor).only("rekord")
        )

        qs = QuerySetSequence(wydawnictwa_ciagle, wydawnictwa_zwarte, patenty)

        if self.q:
            qs = qs.filter(tytul_oryginalny__icontains=self.q)

        qs = self.mixup_querysets(qs)

        return qs


class Dyscyplina_NaukowaAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = Dyscyplina_Naukowa.objects.all()
        if self.q:
            qs = qs.filter(Q(nazwa__icontains=self.q) | Q(kod__icontains=self.q))
        return qs


class Zewnetrzna_Baza_DanychAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = Zewnetrzna_Baza_Danych.objects.all()
        if self.q:
            qs = qs.filter(Q(nazwa__icontains=self.q) | Q(skrot__icontains=self.q))
        return qs


class Dyscyplina_Naukowa_PrzypisanieAutocomplete(autocomplete.Select2ListView):
    def results(self, results):
        """Return data for the 'results' key of the response."""
        return [{"id": _id, "text": value} for _id, value in results]

    def autocomplete_results(self, results):
        return [(x, y) for x, y in results if self.q.lower() in y.lower()]

    def get_list(self):
        autor = self.forwarded.get("autor", None)
        if autor is None:
            return [(None, "Podaj autora")]

        rok = self.forwarded.get("rok", None)
        if rok is None:
            return [(None, "Podaj rok")]
        try:
            rok = int(rok)
        except (TypeError, ValueError):
            return [(None, "Nieprawidłowy rok")]
        if rok < 0 or rok > 9999:
            return [(None, "Nieprawidłowy rok")]

        try:
            ad = Autor_Dyscyplina.objects.get(rok=rok, autor=autor)
        except Autor_Dyscyplina.DoesNotExist:
            return [(None, "Brak przypisania dla roku %s" % rok)]

        res = set()
        for elem in ["dyscyplina_naukowa", "subdyscyplina_naukowa"]:
            id = getattr(ad, "%s_id" % elem)
            if id is not None:
                res.add((id, getattr(ad, elem).nazwa))

        res = list(res)
        res.sort(key=lambda obj: obj[1])

        return res



from rest_framework import viewsets

from api_v1.serializers.system import (
    Charakter_FormalnySerializer,
    Typ_KBNSerializer,
    JezykSerializer,
    Dyscyplina_NaukowaSerializer,
    KonferencjaSerializer,
    Seria_WydawniczaSerializer,
)
from bpp.models import (
    Charakter_Formalny,
    Typ_KBN,
    Jezyk,
    Dyscyplina_Naukowa,
    Konferencja,
    Seria_Wydawnicza,
)


class Seria_WydawniczaViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Seria_Wydawnicza.objects.all()
    serializer_class = Seria_WydawniczaSerializer


class KonferencjaViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Konferencja.objects.all()
    serializer_class = KonferencjaSerializer


class Dyscyplina_NaukowaViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Dyscyplina_Naukowa.objects.all()
    serializer_class = Dyscyplina_NaukowaSerializer


class Charakter_FormalnyViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Charakter_Formalny.objects.all()
    serializer_class = Charakter_FormalnySerializer


class Typ_KBNViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Typ_KBN.objects.all()
    serializer_class = Typ_KBNSerializer


class JezykViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Jezyk.objects.all()
    serializer_class = JezykSerializer



# -*- encoding: utf-8 -*

"""
delete from bpp_autor_jednostka ;delete from bpp_autor;delete from bpp_bppuser_groups; delete from bpp_bppuser;delete from bpp_jednostka;

delete from bpp_zrodlo;

delete from bpp_wydawnictwo_zwarte_autor;delete from bpp_wydawnictwo_zwarte; delete from bpp_wydawnictwo_ciagle_autor; delete from bpp_wydawnictwo_ciagle; delete from bpp_patent_autor; delete from bpp_patent; delete from bpp_praca_doktorska; delete from bpp_praca_habilitacyjna;
"""

from optparse import make_option
import subprocess
import sys

from django.core.management import BaseCommand


class Command(BaseCommand):
    help = 'Uruchamiam import_bpp na wielu CPU'

    def add_arguments(self, parser):
        parser.add_argument("--cpu", action="store", type=int, default=4)

    def handle(self, *args, **options):
        cpus = options['cpu']

        jednowatkowe = ['uzytkownicy', 'korekty', 'clusters']

        for option in ['publikacje']:
            # '['uzytkownicy', 'jednostki', 'autorzy',
            #                'powiazania', 'zrodla', 'korekty', 'publikacje', 'clusters']:
            proc = []
            if option in jednowatkowe:
                ret = subprocess.check_call(
                    [sys.executable, sys.argv[1], 'import_bpp',
                     '--' + option,
                     '--traceback'])
                continue

            else:
                for n in range(cpus):
                    ret = subprocess.Popen(
                        [sys.executable, sys.argv[0], 'import_bpp',
                         '--' + option,
                         '--initial-offset=%s' % n,
                         '--skip=%s' % (cpus-1),
                         '--traceback'])
                    proc.append(ret)

                for elem in proc:
                    elem.wait()

        proc = []
        for n in range(cpus):
            ret = subprocess.Popen(
                [sys.executable, sys.argv[0], 'rebuild_cache',
                 '--initial-offset=%s' % n,
                 '--skip=%s' % (cpus-1),
                 '--traceback'])
            proc.append(ret)

        for elem in proc:
            elem.wait()



import os

import pytest
from django.core.files.base import File
from django.core.files.storage import default_storage
from model_bakery import baker

from integrator2.models import ListaMinisterialnaIntegration

__dirname__ = os.path.dirname(__file__)


def test_file(name):
    return os.path.join(__dirname__, "xls", name)


def test_xls(name):
    return open(test_file(name), "rb")


@pytest.fixture
def lmi_base(db, admin_user):
    lmi = baker.make(ListaMinisterialnaIntegration, owner=admin_user)
    lmi.save()
    return lmi


def _make_lmi(_lmi, fn, fn2):
    pass

    path = default_storage.save(fn2, test_xls(fn))

    _lmi.file = File(default_storage.open(path), name=fn2)
    _lmi.save()
    return _lmi


@pytest.fixture
def lmi(lmi_base):
    return _make_lmi(lmi_base, "lista_a_krotka.xlsx", "a.xlsx")


@pytest.fixture
def lmi_b(lmi_base):
    return _make_lmi(lmi_base, "lista_b.xlsx", "b.xlsx")


@pytest.fixture
def lmi_c(lmi_base):
    return _make_lmi(lmi_base, "lista_c.xlsx", "c.xlsx")



# Generated by Django 3.2.14 on 2022-07-09 23:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("import_dyscyplin", "0020_django32"),
    ]

    operations = [
        migrations.AlterField(
            model_name="import_dyscyplin_row",
            name="dyscyplina",
            field=models.CharField(
                blank=True, db_index=True, max_length=200, null=True
            ),
        ),
        migrations.AlterField(
            model_name="import_dyscyplin_row",
            name="kod_dyscypliny",
            field=models.CharField(blank=True, db_index=True, max_length=20, null=True),
        ),
    ]



import bleach
from django import shortcuts

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from django.http import JsonResponse
from django.http.response import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.defaults import page_not_found, permission_denied, server_error
from sendfile import sendfile

from bpp.models import Uczelnia


def root(request):
    """Zachowanie domyślne: przekieruj nas na pierwszą dostępną w bazie danych
    uczelnię, lub wyświetl komunikat jeżeli nie ma żadnych uczelni wpisanych do
    bazy danych."""
    # TODO: jeżeli będzie więcej, niż jeden obiekt Uczelnia...?
    uczelnia = Uczelnia.objects.only("slug").first()

    if uczelnia is None:
        return shortcuts.render(request, "browse/brak_uczelni.html")

    return shortcuts.redirect(
        reverse("bpp:browse_uczelnia", args=(uczelnia.slug,)), permanent=True
    )


def favicon(request):
    try:
        fn = Uczelnia.objects.get(pk=1).favicon_ico
    except Uczelnia.DoesNotExist:
        return HttpResponse("create Uczelnia object first")

    try:
        return sendfile(request, fn.path)
    except ValueError:
        return HttpResponse("icon image is not set")
        # raise Http404


@csrf_exempt
def update_multiseek_title(request):
    v = request.POST.get("value")
    if not v or not len(v):
        v = ""
    from django.conf import settings

    v = bleach.clean(
        v.replace("\r\n", "\n").replace("\n", "<br/>"),
        tags=list(getattr(settings, "ALLOWED_TAGS", [])) + ["hr", "p", "br"],
    )
    request.session["MULTISEEK_TITLE"] = v
    return JsonResponse(v, safe=False)


# !/usr/bin/env python
# vim:ts=4:sw=4:et:ft=python
#
# Caching decorator for Django /jsi18n/
# http://wtanaka.com/django/jsi18ncache
#
# Copyright (C) 2009 Wesley Tanaka <http://wtanaka.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime

TEN_YEARS = datetime.timedelta(days=3650)


def javascript_catalog(request, domain="djangojs", packages=None):
    import django.views.i18n

    response = django.views.i18n.javascript_catalog(
        request, domain=domain, packages=packages
    )
    from django.utils.translation import check_for_language

    if (
        request.GET
        and "language" in request.GET
        and check_for_language(request.GET["language"])
    ):
        expires = datetime.datetime.now() + TEN_YEARS
        response["Expires"] = expires.strftime("%a, %d %b %Y %H:%M:%S GMT")
        response["Cache-Control"] = "public"
    return response


def handler404(request, exception):
    return page_not_found(request, exception, "404.html")


def handler403(request, exception=None):
    return permission_denied(request, "403.html")


def handler500(request):
    return server_error(request, template_name="50x.html")



from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse
from import_export.admin import ExportMixin
from import_export.signals import post_export

from .formats import PrettyXLSX

from django.contrib.admin.options import IncorrectLookupParameters


class EksportDanychMixin(ExportMixin):
    """Klasa do eksportu danych, bazująca na django-import-export,

    włącza pozwolenie na eksport, gdy na liście admina changelist jest wyświetlane
     poniżej max_allowed_items. Gdy elementów jest więcej, eksport jest niedozwolony,

    eksportuje wyłącznie do XLSX w wersji upiększonej -- automatyczne szerokości wierszy,
    format tableki itp"""

    max_allowed_export_items = None

    def has_export_permission(self, request):
        try:
            cl = self.get_changelist_instance(request)
        except IncorrectLookupParameters:
            return

        max_allowed_export_items = self.max_allowed_export_items
        if max_allowed_export_items is None:
            max_allowed_export_items = getattr(
                settings, "BPP_MAX_ALLOWED_EXPORT_ITEMS", 100
            )

        if cl.result_count < max_allowed_export_items:
            return True

    def export_action(self, request, *args, **kwargs):
        """Eksportuj zawsze do XLSX z upiększaniem -- autorozmiar wierszy + format jako tabela."""
        if not self.has_export_permission(request):
            raise PermissionDenied

        file_format = PrettyXLSX()

        queryset = self.get_export_queryset(request)
        export_data = self.get_export_data(file_format, queryset, request=request)
        content_type = file_format.get_content_type()

        response = HttpResponse(export_data, content_type=content_type)
        response["Content-Disposition"] = 'attachment; filename="{}"'.format(
            self.get_export_filename(request, queryset, file_format),
        )

        post_export.send(sender=None, model=self.model)
        return response



# Generated by Django 3.0.14 on 2021-06-06 23:09

from django.db import migrations

import bpp.models.dyscyplina_naukowa
import bpp.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0275_jednostka_hier"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="pokazuj_raport_uczelni",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="never",
                max_length=50,
                verbose_name="Pokazuj raport uczelni",
            ),
        ),
    ]



# Generated by Django 3.0.14 on 2021-11-09 23:02

import django.db.models.deletion
from django.db import migrations, models

import ewaluacja2021.models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0309_auto_20211110_0000"),
        ("ewaluacja2021", "0006_auto_20211110_0000"),
    ]

    operations = [
        migrations.AlterField(
            model_name="zamowienienaraport",
            name="dyscyplina_naukowa",
            field=models.ForeignKey(
                limit_choices_to=ewaluacja2021.models.dyscypliny_naukowe_w_bazie,
                on_delete=django.db.models.deletion.CASCADE,
                to="bpp.Dyscyplina_Naukowa",
            ),
        ),
    ]



import pytest
from django.core.exceptions import ValidationError

from bpp.models import (
    Autor_Dyscyplina,
    Typ_Odpowiedzialnosci,
    Wydawnictwo_Ciagle_Autor,
    parse_informacje_as_dict,
    wez_zakres_stron,
)


@pytest.mark.django_db
def test_baza_modelu_odpowiedzialnosci_zapisywanie(
    wydawnictwo_ciagle,
    autor_jan_nowak,
    rok,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    typy_odpowiedzialnosci,
    db,
):
    wydawnictwo_ciagle.rok = rok
    wydawnictwo_ciagle.save()

    ad = Autor_Dyscyplina.objects.create(
        autor=autor_jan_nowak,
        rok=rok,
        dyscyplina_naukowa=dyscyplina1,
        procent_dyscypliny=50,
    )

    wca = Wydawnictwo_Ciagle_Autor.objects.create(
        rekord=wydawnictwo_ciagle,
        autor=autor_jan_nowak,
        jednostka=jednostka,
        typ_odpowiedzialnosci=Typ_Odpowiedzialnosci.objects.get(skrot="aut."),
        zapisany_jako="Foobar",
        dyscyplina_naukowa=None,
    )
    wca.save()

    wca.clean()

    wca.dyscyplina_naukowa = dyscyplina2
    with pytest.raises(ValidationError):
        wca.clean()

    wca.dyscyplina_naukowa = dyscyplina1
    wydawnictwo_ciagle.rok = 50
    wydawnictwo_ciagle.save()
    with pytest.raises(ValidationError):
        wca.clean()

    wydawnictwo_ciagle.rok = rok
    wydawnictwo_ciagle.save()
    wca.clean()

    ad.dyscyplina_naukowa = dyscyplina2
    ad.subdyscyplina_naukowa = dyscyplina1
    ad.save()
    wca.clean()


@pytest.mark.django_db
def test_baza_modelu_odpowiedzialnosci_autorow_dyscyplina_okresl_dyscypline(
    wydawnictwo_ciagle,
    jednostka,
    autor_jan_kowalski,
    dyscyplina1,
    dyscyplina2,
    typy_odpowiedzialnosci,
    rok,
):
    wca = wydawnictwo_ciagle.dodaj_autora(
        autor_jan_kowalski, jednostka, zapisany_jako="Kowalski"
    )
    assert wca.okresl_dyscypline() is None

    Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    assert wca.okresl_dyscypline() is None

    wca.dyscyplina_naukowa = dyscyplina2
    wca.save()
    assert wca.okresl_dyscypline() == dyscyplina2

    wca.dyscyplina_naukowa = None
    wca.save()
    assert wca.okresl_dyscypline() is None


@pytest.mark.django_db
@pytest.mark.parametrize(
    "input,expected",
    [
        ("123", "123"),
        ("123 s.", "123"),
        ("123-123", "123-123"),
        ("s. 35", "35"),
        ("ss. 90", "90"),
        ("ss. 190", "190"),
        ("ss. 290", "290"),
        ("s. 10-20", "10-20"),
        ("tia", None),
        ("s. e27-e53", "e27-e53"),
        ("[b.pag.]", "brak"),
        ("[b. pag.]", "brak"),
        ("[b. pag]", "brak"),
        ("s. 132-153.", "132-153"),
        ("s. 143.", "143"),
        ("aosidfjoaisd fjoiasdjf s. 132-153.", "132-153"),
        ("      s. 143.", "143"),
        ("      s.          143.", "143"),
        ("s. P-29.", "P-29"),
        ("s. PP-4.", "PP-4"),
        # ("s. P-27-P-28.", "P-27-P-28"), # to się nie uda
        ("s. xiii-xiv.", "xiii-xiv"),
    ],
)
def test_eksport_pbn_zakres_stron(input, expected):
    assert wez_zakres_stron(input) == expected


@pytest.mark.django_db
def test_eksport_pbn_zakres_stron_pole(wydawnictwo_ciagle):
    """Przetestuj że w sytuacji, gdy jest wypełnione pole 'Strony', to
    jego wartość idzie do eksportu"""
    wydawnictwo_ciagle.szczegoly = "s. 35"
    wydawnictwo_ciagle.strony = "44-44"
    ret = wydawnictwo_ciagle.zakres_stron()
    assert ret == "44-44"

    wydawnictwo_ciagle.strony = None
    ret = wydawnictwo_ciagle.zakres_stron()
    assert ret == "35"


@pytest.mark.django_db
def test_eksport_pbn_get_issue(wydawnictwo_ciagle):
    wydawnictwo_ciagle.nr_zeszytu = "10"
    assert wydawnictwo_ciagle.numer_wydania() == "10"

    wydawnictwo_ciagle.nr_zeszytu = None
    wydawnictwo_ciagle.informacje = "1993 z. 5"
    assert wydawnictwo_ciagle.numer_wydania() == "5"


@pytest.mark.django_db
def test_eksport_pbn_get_volume(wydawnictwo_ciagle):
    wydawnictwo_ciagle.tom = "10"
    assert wydawnictwo_ciagle.numer_tomu() == "10"

    wydawnictwo_ciagle.tom = None
    wydawnictwo_ciagle.informacje = "1992 vol. 5"
    assert wydawnictwo_ciagle.numer_tomu() == "5"


@pytest.mark.parametrize(
    "input,exp_rok,exp_tom,exp_nr",
    [
        ("1960", "1960", None, None),
        ("1960 t. 8", "1960", "8", None),
        ("1960 t 8", "1960", "8", None),
        ("1960 nr 2", "1960", None, "2"),
        ("1960 nr. 2", "1960", None, "2"),
        ("1960 t. 8 nr 2", "1960", "8", "2"),
        ("1960 T. 8 nr 2", "1960", "8", "2"),
        ("1960 T.8nr2", "1960", "8", "2"),
        ("1960 T.8 nr 2", "1960", "8", "2"),
        ("2018 Vol.77 suppl.2", "2018", "77", "suppl.2"),
        ("2020 T. [59] supl.", "2020", "59", "supl."),
        ("2020 Vol.61 no.7-12 supl.5", "2020", "61", "7-12 supl.5"),
        ("2020 Vol. A74 [suppl.]", "2020", "A74", "[suppl.]"),
        ("2020 voL 54 SuPPl. 45", "2020", "54", "SuPPl. 45"),
        ("2018 Vol.48 suppl.", "2018", "48", "suppl."),
        ("2020 Vol.60 supl.3", "2020", "60", "supl.3"),
        ("2020 supl. 2/1", "2020", None, "supl. 2/1"),
        ("2018 Vol.72 no.13 suppl.B", "2018", "72", "13 suppl.B"),
        ("2018 Vol.35 e-suppl.56", "2018", "35", "e-suppl.56"),
        ("2020 Vol.61 no.2 suppl.2", "2020", "61", "2 suppl.2"),
        ("2020 Vol.15 no.5 suppl.", "2020", "15", "5 suppl."),
        ("1998 Vol.4 suppl.2, fig., bibliogr. 11 poz., summ.", "1998", "4", "suppl.2"),
    ],
)
def test_parse_informacje(input, exp_rok, exp_tom, exp_nr):
    res = parse_informacje_as_dict(input)
    assert res.get("rok") == exp_rok
    assert res.get("tom") == exp_tom
    assert res.get("numer") == exp_nr



import time

from django.core.cache import cache
from django.db.models.signals import post_save
from django.dispatch.dispatcher import receiver

from bpp.models.struktura import Uczelnia


class NiezdefiniowanaUczelnia:
    pk = None
    nazwa = "[niezdefiniowana uczelnia]"
    nazwa_dopelniacz = "[niezdefiniowanej uczelni]"
    slug = "niezdefiniowana-uczelnia"
    podpowiadaj_dyscypliny = False
    skrot = "NdU"

    def __getattr__(self, item):
        if item.startswith("pokazuj_"):
            return False
        return super().__getattr__(item)

    def sprawdz_uprawnienie(self, *args, **kw):
        return False


BRAK_UCZELNI = {
    "uczelnia": NiezdefiniowanaUczelnia,
}


def uczelnia(request):
    timeout, value = cache.get(b"bpp_uczelnia", (0, None))

    if value is not None:
        if time.time() < timeout:
            return value

    u = Uczelnia.objects.get_for_request(request)
    if u is None:
        return BRAK_UCZELNI

    value = {"uczelnia": u}
    cache.set(b"bpp_uczelnia", (time.time() + 3600, value))
    return value


@receiver(post_save, sender=Uczelnia)
def remove_cache_key(*args, **kw):
    cache.delete(b"bpp_uczelnia")



# -*- encoding: utf-8 -*-
# save me as yourapp/management/commands/upload_file_to_model.py

from argparse import FileType
from pathlib import Path

from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand
from django.core.files.base import File

class Command(BaseCommand):
    help = 'Uploads a file to a given field in a given model'

    def add_arguments(self, parser):
        parser.add_argument("app")
        parser.add_argument("model")
        parser.add_argument("pk")
        parser.add_argument("field")
        parser.add_argument("path", type=FileType('rb'))

    def handle(self, *args, **options):
        obj = ContentType.objects.get_by_natural_key(
            options['app'].lower(),
            options['model'].lower()
        ).get_object_for_this_type(pk=options['pk'])

        try:
            field = getattr(obj, options['field'])
        except AttributeError as e:
            fields = [field.name for field in obj._meta.get_fields()]
            fields = ", ".join(fields)
            e.args = (e.args[0] + f". Available names: {fields}", )
            raise e

        field.save(
            name=Path(options['path'].name).name,
            content=File(options['path']))
        obj.save()



# -*- encoding: utf-8 -*-

from django.core.management import BaseCommand
from django.db import transaction
from bpp.models import Opi_2012_Tytul_Cache


class Command(BaseCommand):
    help = "Keszuje tytuly prac"

    @transaction.atomic
    def handle(self, *args, **options):
        Opi_2012_Tytul_Cache.objects.rebuild()


# Generated by Django 2.1.10 on 2019-10-09 06:02

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0180_merge_20190910_2236'),
    ]

    operations = [
        migrations.CreateModel(
            name='Cache_Punktacja_Autora_Sum',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('pkdaut', models.DecimalField(decimal_places=4, max_digits=20)),
                ('slot', models.DecimalField(decimal_places=4, max_digits=20)),
                ('pkdautslot', models.FloatField()),
                ('pkdautsum', models.FloatField()),
                ('pkdautslotsum', models.FloatField()),
            ],
            options={
                'db_table': 'bpp_temporary_cpaq',
                'ordering': ('autor', 'dyscyplina', '-pkdautslot'),
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Cache_Punktacja_Autora_Sum_Gruop',
            fields=[
                ('autor', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, primary_key=True, serialize=False, to='bpp.Autor')),
                ('pkdautsum', models.FloatField()),
                ('pkdautslotsum', models.FloatField()),
            ],
            options={
                'db_table': 'bpp_temporary_cpasg',
                'ordering': ('autor', 'dyscyplina'),
                'managed': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-23 21:18
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0101_openaccess_w_multiseek'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='patent',
            name='afiliowana',
        ),
        migrations.RemoveField(
            model_name='praca_doktorska',
            name='afiliowana',
        ),
        migrations.RemoveField(
            model_name='praca_habilitacyjna',
            name='afiliowana',
        ),
        migrations.RemoveField(
            model_name='wydawnictwo_ciagle',
            name='afiliowana',
        ),
        migrations.RemoveField(
            model_name='wydawnictwo_zwarte',
            name='afiliowana',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-11-08 11:59
from __future__ import unicode_literals

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0111_autorzy_afiliacja_zatrudniony'),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0112_fix_refresh_cache")
        ),

    ]



from rest_framework import serializers

# Serializers define the API representation.
from api_v1.serializers.util import AbsoluteUrlSerializerMixin, ChoicesSerializerField
from bpp.models import Zrodlo, Rodzaj_Zrodla


class Rodzaj_ZrodlaSerializer(serializers.ModelSerializer):
    class Meta:
        model = Rodzaj_Zrodla
        fields = [
            "id",
            "nazwa",
        ]


class ZrodloSerializer(
    AbsoluteUrlSerializerMixin, serializers.HyperlinkedModelSerializer
):
    rodzaj = serializers.HyperlinkedRelatedField(
        view_name="api_v1:rodzaj_zrodla-detail", read_only=True
    )

    jezyk = serializers.HyperlinkedRelatedField(
        view_name="api_v1:jezyk-detail", read_only=True
    )

    zasieg = serializers.StringRelatedField()

    openaccess_licencja = serializers.StringRelatedField()
    openaccess_tryb_dostepu = serializers.StringRelatedField()

    class Meta:
        model = Zrodlo
        fields = [
            "id",
            "nazwa",
            "skrot",
            "rodzaj",
            "nazwa_alternatywna",
            "skrot_nazwy_alternatywnej",
            "zasieg",
            "www",
            "doi",
            "poprzednia_nazwa",
            "jezyk",
            "wydawca",
            #
            "openaccess_tryb_dostepu",
            "openaccess_licencja",
            #
            "issn",
            "e_issn",
            #
            "absolute_url",
            #
            "ostatnio_zmieniony",
        ]



# -*- encoding: utf-8 -*-

import time

from django import template

from django_bpp.version import VERSION

register = template.Library()


@register.simple_tag
def bpp_version():
    return VERSION

@register.simple_tag
def bpp_localtime():
    return time.ctime()


# def pobierz_skasowane_prace(client: PBNClient):
#     pobierz_mongodb(
#         client.get_publications(status=DELETED, page_size=50),
#         Publication,
#         pbar_label="pobierz_skasowane_prace",
#     )


class ThreadedPageGetter:
    def pool_init(self, _client, _data):
        import django

        django.setup()

        self.client = _client
        self.data = _data

    def process_element(self, elem):
        raise NotImplementedError

    def get_single_page(self, n):
        for elem in self.data.fetch_page(n):
            self.process_element(elem)


class ThreadedMongoDBSaver(ThreadedPageGetter):
    def process_element(self, elem):
        if not hasattr(self, "zapisz_mongodb"):
            from pbn_api.integrator import zapisz_mongodb

            self.zapisz_mongodb = zapisz_mongodb
        self.zapisz_mongodb(elem, self.pbn_api_klass)


def threaded_page_getter(
    client,
    data,
    klass=ThreadedPageGetter,
    no_threads=12,
    label="getting...",
    method="threads",
):
    """Odświeża wszystkie publikacje, które są w lokalnej tabeli."""

    getter = klass()

    if method == "processes":
        import multiprocessing

        from pbn_api.integrator import (
            _bede_uzywal_bazy_danych_z_multiprocessing_z_django,
        )

        _bede_uzywal_bazy_danych_z_multiprocessing_z_django()

        pool = multiprocessing.get_context("fork").Pool(
            processes=no_threads, initializer=getter.pool_init, initargs=(client, data)
        )

    elif method == "threads":
        from multiprocessing.dummy import Pool

        pool = Pool(
            processes=no_threads, initializer=getter.pool_init, initargs=(client, data)
        )

    from bpp.util import pbar

    for _ in pbar(
        pool.imap_unordered(
            getter.get_single_page,
            range(0, data.total_pages),
        ),
        data.total_pages,
        label=label,
    ):
        pass

    pool.close()
    pool.join()



import sys

import pytest

from bpp.templatetags.prace import close_tags


@pytest.mark.parametrize(
    "i,o",
    [
        ("test<x", "test<x/>"),
        ("test<test", "test<test/>"),
        ("test<strong>", "test<strong/>"),
        (None, None),
        ("", ""),
    ],
)
def test_close_tags(i, o):
    assert close_tags(i) == o


@pytest.mark.skipif(sys.platform == "darwin", reason="lxml behaviour differences")
@pytest.mark.parametrize(
    "i,o",
    [
        ("test<", "test&lt;"),
    ],
)
def test_close_tags_on_darwin(i, o):
    assert close_tags(i) == o



# -*- coding: utf-8 -*-


from django.db import migrations, models
from django.conf import settings
import django_extensions.db.fields
import django_extensions.db.fields.json
from django.db.models import CASCADE


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Report',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('uid', models.UUIDField(unique=True, editable=False, blank=True)),
                ('function', models.TextField()),
                ('arguments', django_extensions.db.fields.json.JSONField(null=True, blank=True)),
                ('ordered_on', models.DateTimeField(auto_now_add=True)),
                ('file', models.FileField(null=True, upload_to=b'report', blank=True)),
                ('started_on', models.DateTimeField(null=True, blank=True)),
                ('finished_on', models.DateTimeField(null=True, blank=True)),
                ('progress', models.FloatField(default=0.0, null=True, blank=True)),
                ('error', models.BooleanField(default=False)),
                ('traceback', models.TextField(max_length=10240, null=True, blank=True)),
                ('ordered_by', models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=CASCADE)),
            ],
            options={
                'ordering': ['-ordered_on'],
            },
        ),
    ]



# -*- encoding: utf-8 -*-

import pymed
from django.conf import settings
from django.http import JsonResponse
from django.views.generic.base import View

from bpp.views.api.const import (
    PUBMED_BRAK_PARAMETRU,
    PUBMED_PO_TYTULE_BRAK,
    PUBMED_PO_TYTULE_WIELE,
    PUBMED_TITLE_MULTIPLE,
    PUBMED_TITLE_NONEXISTENT,
)

from django_bpp.version import VERSION


def get_data_from_ncbi(title):
    pubmed = pymed.PubMed(tool=f"BPP {VERSION}", email=settings.ADMINS[0][0])
    title = (
        title.lower()
        .replace(" and ", " ")
        .replace(" in ", " ")
        .replace(" of ", " ")
        .replace(" the ", " ")
    )

    while title.find("  ") != -1:
        title = title.replace("  ", " ")

    results = list(pubmed.query(title, max_results=2))
    return results


def extract_data(x: "pymed.api.PubMedArticle"):
    return {
        "pubmed_id": x.pubmed_id,
        "doi": x.doi,
        "pmc_id": x.pmc_id,
        "title": x.title,
    }


class GetPubmedIDView(View):
    def post(self, request, *args, **kw):
        tytul = request.POST.get("t", "").strip()[:1024]

        if not tytul:
            return JsonResponse({"error": PUBMED_BRAK_PARAMETRU})

        if tytul == PUBMED_TITLE_NONEXISTENT:
            # ten warunek wykorzystywany jest przez testy integracyjne, nue ruszać go
            results = []
        elif tytul == PUBMED_TITLE_MULTIPLE:
            # ten warunek wykorzystywany jest przez testy integracyjne, nie ruszać go
            results = [None, None]
        else:
            results = get_data_from_ncbi(tytul)

        if len(results) == 0:
            return JsonResponse({"error": PUBMED_PO_TYTULE_BRAK})

        if len(results) > 1:
            return JsonResponse({"error": PUBMED_PO_TYTULE_WIELE})

        return JsonResponse(extract_data(results[0]))



from pbn_api.admin.base import BaseMongoDBAdmin
from pbn_api.admin.filters import OdpowiednikAutoraWBPPFilter
from pbn_api.models import Scientist

from django.contrib import admin

from bpp.admin.filters import OrcidObecnyFilter


@admin.register(Scientist)
class ScientistAdmin(BaseMongoDBAdmin):
    show_full_result_count = False
    list_display = [
        "lastName",
        "name",
        "qualifications",
        "polonUid",
        "orcid",
        "currentEmploymentsInstitutionDisplayName",
        "mongoId",
        "from_institution_api",
        "rekord_w_bpp",
    ]

    search_fields = [
        "mongoId",
        "lastName",
        "name",
        "orcid",
    ]

    fields = BaseMongoDBAdmin.fields + [
        "from_institution_api",
    ]
    readonly_fields = BaseMongoDBAdmin.readonly_fields + ["from_institution_api"]
    list_filter = [
        OdpowiednikAutoraWBPPFilter,
        OrcidObecnyFilter,
        "from_institution_api",
        "qualifications",
    ] + BaseMongoDBAdmin.list_filter



import json
from collections import namedtuple

from django.test import TestCase

from bpp.models import Typ_Odpowiedzialnosci
from bpp.models.zrodlo import Punktacja_Zrodla
from bpp.tests.util import CURRENT_YEAR, any_autor, any_habilitacja, any_zrodlo
from bpp.views.api import (
    PunktacjaZrodlaView,
    RokHabilitacjiView,
    UploadPunktacjaZrodlaView,
)

FakeRequest = namedtuple("FakeRequest", ["POST"])


class TestRokHabilitacjiView(TestCase):
    # fixtures = ['charakter_formalny.json', 'typ_odpowiedzialnosci.json']

    def test_rokhabilitacjiview(self):
        Typ_Odpowiedzialnosci.objects.get_or_create(skrot="aut.", nazwa="autor")

        a = any_autor()
        h = any_habilitacja(tytul_oryginalny="Testowa habilitacja", rok=CURRENT_YEAR)
        h.autor = a
        h.save()

        request = FakeRequest({"autor_pk": a.pk})

        rhv = RokHabilitacjiView()

        res = rhv.post(request)
        self.assertContains(res, str(CURRENT_YEAR), status_code=200)
        self.assertEqual(json.loads(res.content)["rok"], CURRENT_YEAR)

        h.delete()
        res = rhv.post(request)
        self.assertContains(res, "Habilitacja", status_code=404)

        a.delete()
        res = rhv.post(request)
        self.assertContains(res, "Autor", status_code=404)


class TestPunktacjaZrodlaView(TestCase):
    def test_punktacjazrodlaview(self):
        z = any_zrodlo()
        Punktacja_Zrodla.objects.create(zrodlo=z, rok=CURRENT_YEAR, impact_factor=50)

        res = PunktacjaZrodlaView().post(None, z.pk, CURRENT_YEAR)
        analyze = json.loads(res.content.decode(res.charset))
        self.assertEqual(analyze["impact_factor"], "50.000")

        res = PunktacjaZrodlaView().post(None, z.pk, CURRENT_YEAR + 100)
        self.assertContains(res, "Rok", status_code=404)

    def test_punktacjazrodlaview_404(self):
        res = PunktacjaZrodlaView().post(None, 1, CURRENT_YEAR)
        self.assertContains(res, "Zrodlo", status_code=404)


class TestUploadPunktacjaZrodlaView(TestCase):
    def test_upload_punktacja_zrodla_404(self):
        res = UploadPunktacjaZrodlaView().post(None, 1, CURRENT_YEAR)
        self.assertContains(res, "Zrodlo", status_code=404)

    def test_upload_punktacja_zrodla_simple(self):
        z = any_zrodlo()
        fr = FakeRequest(dict(impact_factor="50.00"))
        UploadPunktacjaZrodlaView().post(fr, z.pk, CURRENT_YEAR)
        self.assertEqual(Punktacja_Zrodla.objects.count(), 1)
        self.assertEqual(Punktacja_Zrodla.objects.all()[0].impact_factor, 50)

    def test_upload_punktacja_zrodla_overwrite(self):
        z = any_zrodlo()
        Punktacja_Zrodla.objects.create(rok=CURRENT_YEAR, zrodlo=z, impact_factor=50)
        fr = FakeRequest(dict(impact_factor="60.00", punkty_kbn="60"))
        res = UploadPunktacjaZrodlaView().post(fr, z.pk, CURRENT_YEAR)
        self.assertContains(res, "exists", status_code=200)

        fr = FakeRequest(dict(impact_factor="60.00", overwrite="1"))
        UploadPunktacjaZrodlaView().post(fr, z.pk, CURRENT_YEAR)
        self.assertEqual(Punktacja_Zrodla.objects.count(), 1)
        self.assertEqual(Punktacja_Zrodla.objects.all()[0].impact_factor, 60)



# Generated by Django 3.0.14 on 2021-10-01 20:26

import denorm.fields
from django.db import migrations

import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0296_nulltest_szablonopisu"),
    ]

    operations = [
        migrations.AddField(
            model_name="wydawca",
            name="ile_aliasow",
            field=denorm.fields.CountField("wydawca_set", default=0),
        ),
        migrations.AddField(
            model_name="wydawca",
            name="lista_poziomow",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True, editable=False, null=True
            ),
        ),
    ]



""" Tablib - XLSX Support.
"""

from io import BytesIO

from openpyxl.workbook import Workbook
from tablib.formats._xlsx import XLSXFormat

from bpp.util import (
    worksheet_columns_autosize,
    worksheet_create_table,
    worksheet_create_urls,
)


class PrettyXLSXFormat(XLSXFormat):
    @classmethod
    def export_set(cls, dataset, freeze_panes=True):
        """Returns XLSX representation of Dataset."""

        wb = Workbook()
        ws = wb.worksheets[0]
        ws.title = dataset.title if dataset.title else "Eksport z BPP"

        cls.dset_sheet(dataset, ws, freeze_panes=freeze_panes)

        worksheet_create_urls(ws)
        worksheet_columns_autosize(ws)
        worksheet_create_table(ws)

        if freeze_panes:
            # Zablokuj pierwsze 2 kolumny (ID, tytuł) oraz wiersz nagłowka
            ws.freeze_panes = ws["F2"]

        stream = BytesIO()
        wb.save(stream)
        return stream.getvalue()



# Generated by Django 3.0.11 on 2021-01-25 22:30

from django.db import migrations, models

import bpp.models.uczelnia


class Migration(migrations.Migration):

    dependencies = [
        ("raport_slotow", "0005_auto_20210125_0256"),
    ]

    operations = [
        migrations.AlterField(
            model_name="raportslotowuczelnia",
            name="do_roku",
            field=models.IntegerField(
                default=bpp.models.uczelnia.UczelniaManager.do_roku_default
            ),
        ),
        migrations.AlterField(
            model_name="raportslotowuczelniawiersz",
            name="avg",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=8,
                null=True,
                verbose_name="Średnio punktów dla autora na slot",
            ),
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-07 22:32

from django.db import migrations, models

import django.contrib.postgres.fields

import bpp.models.uczelnia


class Migration(migrations.Migration):

    dependencies = [
        ("raport_slotow", "0011_auto_20210315_0141"),
    ]

    operations = [
        migrations.CreateModel(
            name="RaportEwaluacjaUpowaznieniaView",
            fields=[
                (
                    "id",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.PositiveIntegerField(),
                        primary_key=True,
                        serialize=False,
                        size=4,
                    ),
                ),
            ],
            options={
                "db_table": "bpp_ewaluacja_upowaznienia_view",
                "managed": False,
            },
        ),
        migrations.AlterField(
            model_name="raportslotowuczelnia",
            name="do_roku",
            field=models.IntegerField(
                default=bpp.models.uczelnia.UczelniaManager.do_roku_default
            ),
        ),
    ]



import django_filters
from rest_framework import viewsets

from api_v1.serializers.autor import (
    AutorSerializer,
    Funkcja_AutoraSerializer,
    TytulSerializer,
    Autor_JednostkaSerializer,
)
from bpp.models import Autor, Funkcja_Autora, Tytul, Autor_Jednostka


class AutorFilterSet(django_filters.rest_framework.FilterSet):
    ostatnio_zmieniony = django_filters.DateTimeFromToRangeFilter("ostatnio_zmieniony")

    class Meta:
        fields = ["ostatnio_zmieniony"]
        model = Autor


class AutorViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Autor.objects.all()
    serializer_class = AutorSerializer
    filterset_class = AutorFilterSet


class Funkcja_AutoraViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Funkcja_Autora.objects.all()
    serializer_class = Funkcja_AutoraSerializer


class Autor_JednostkaViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Autor_Jednostka.objects.all()
    serializer_class = Autor_JednostkaSerializer


class TytulViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Tytul.objects.all()
    serializer_class = TytulSerializer



import sys

from django import forms
from djangoql.admin import DjangoQLSearchMixin
from mptt.admin import DraggableMPTTAdmin

from ..models.struktura import Jednostka, Jednostka_Wydzial
from .core import BaseBppAdminMixin, RestrictDeletionToAdministracjaGroupMixin
from .filters import PBN_UID_IDObecnyFilter
from .helpers import ADNOTACJE_FIELDSET, LimitingFormset, ZapiszZAdnotacjaMixin

from django.contrib import admin

from django.utils.html import format_html

from bpp.models import Autor_Jednostka, Uczelnia


class Jednostka_WydzialInline(admin.TabularInline):
    model = Jednostka_Wydzial
    extra = 1


class Autor_JednostkaForm(forms.ModelForm):
    model = Autor_Jednostka

    class Meta:
        fields = ["autor", "rozpoczal_prace", "zakonczyl_prace", "funkcja"]


class Autor_JednostkaInline(admin.TabularInline):
    form = Autor_JednostkaForm
    model = Autor_Jednostka
    readonly_fields = ["autor"]
    formset = LimitingFormset
    extra = 0


class JednostkaAdmin(
    DjangoQLSearchMixin,
    RestrictDeletionToAdministracjaGroupMixin,
    ZapiszZAdnotacjaMixin,
    BaseBppAdminMixin,
    DraggableMPTTAdmin,
):
    djangoql_completion_enabled_by_default = False
    djangoql_completion = True

    change_list_template = "admin/grappelli_mptt_change_list.html"

    list_display_links = ["indented_title"]

    list_display = [
        "tree_actions",
        "indented_title",
        "skrot",
        "parent_nazwa",
        "wydzial_skrot",
        "widoczna",
        "wchodzi_do_raportow",
        "skupia_pracownikow",
        "pbn_uid_id",
    ]

    list_select_related = [
        "wydzial",
    ]
    fields = None
    list_filter = (
        "wydzial",
        "widoczna",
        "wchodzi_do_raportow",
        "skupia_pracownikow",
        "zarzadzaj_automatycznie",
        PBN_UID_IDObecnyFilter,
    )
    search_fields = ["nazwa", "skrot", "wydzial__nazwa"]

    inlines = (
        Jednostka_WydzialInline,
        Autor_JednostkaInline,
    )

    autocomplete_fields = ["pbn_uid"]

    readonly_fields = ["wydzial", "aktualna", "ostatnio_zmieniony"]
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "nazwa",
                    "skrot",
                    "uczelnia",
                    "wydzial",
                    "parent",
                    "aktualna",
                    "pbn_id",
                    "pbn_uid",
                    "opis",
                    "widoczna",
                    "wchodzi_do_raportow",
                    "skupia_pracownikow",
                    "zarzadzaj_automatycznie",
                    "email",
                    "www",
                ),
            },
        ),
        ADNOTACJE_FIELDSET,
    )

    def get_changeform_initial_data(self, request):
        # Zobacz na komentarz do Jednostka.uczelnia.default
        data = super().get_changeform_initial_data(request)
        if "uczelnia" not in data:
            data["uczelnia"] = Uczelnia.objects.first()
        return data

    def changelist_view(self, request, *args, **kwargs):
        self.request = request
        return super().changelist_view(request, *args, **kwargs)

    def indented_title(self, item):
        """
        Generate a short title for an object, indent it depending on
        the object's depth in the hierarchy.
        """

        # Wysuwaj nazwę jednostki wyłacznie w przypadku, gdy nie filtrujemy listy
        if not self.request.GET.keys():
            return super().indented_title(item)

        return format_html(
            "<div>{}</div>",
            item,
        )

    indented_title.short_description = "Nazwa jednostki"

    def wydzial_skrot(self, item):
        if item.wydzial_id:
            return item.wydzial.skrot

    wydzial_skrot.short_description = "Wydział"

    def parent_nazwa(self, item):
        if item.parent_id:
            return item.parent.nazwa

    parent_nazwa.short_description = "Jednostka nadrzędna"

    def get_list_display(self, request):
        if request.GET.keys():
            return self.list_display[1:]
        return self.list_display

    def get_list_per_page(self):
        from django.db import connection

        if "bpp_uczelnia" not in connection.introspection.table_names():
            return BaseBppAdminMixin.list_per_page

        req = None
        if hasattr(self, "request"):
            req = self.request

        uczelnia = Uczelnia.objects.get_for_request(req)

        if uczelnia is None:
            return BaseBppAdminMixin.list_per_page

        if uczelnia.sortuj_jednostki_alfabetycznie:
            return BaseBppAdminMixin.list_per_page

        return sys.maxsize

    list_per_page = property(get_list_per_page)


admin.site.register(Jednostka, JednostkaAdmin)



from django.db import models
from django.template.loader import get_template

from django.contrib.contenttypes.models import ContentType

from django.utils.functional import cached_property


class SzablonDlaOpisuBibliograficznegoManager(models.Manager):
    def get_for_model(self, model):
        model = ContentType.objects.get_for_model(model)
        try:
            return self.get(model=model).template.name
        except SzablonDlaOpisuBibliograficznego.DoesNotExist:
            try:
                return self.get(model=None).template.name
            except SzablonDlaOpisuBibliograficznego.DoesNotExist:
                return

    @cached_property
    def all_templated_models(self):
        from bpp.models.patent import Patent
        from bpp.models.praca_doktorska import Praca_Doktorska
        from bpp.models.praca_habilitacyjna import Praca_Habilitacyjna
        from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
        from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte

        return [
            Wydawnictwo_Ciagle,
            Wydawnictwo_Zwarte,
            Praca_Doktorska,
            Praca_Habilitacyjna,
            Patent,
        ]

    def get_models_for_template(self, template):
        """Zwraca listę wszystkich modeli wykorzystujących dany szablon."""

        res = list(
            self.filter(template=template).values_list("model", flat=True).distinct()
        )
        if None in res:
            return self.all_templated_models
        return [ContentType.objects.get_for_id(id).model_class() for id in res]


class SzablonDlaOpisuBibliograficznego(models.Model):
    objects = SzablonDlaOpisuBibliograficznegoManager()

    model = models.OneToOneField(
        "contenttypes.ContentType",
        on_delete=models.CASCADE,
        limit_choices_to=models.Q(
            app_label="bpp",
            model__in=[
                "wydawnictwo_ciagle",
                "wydawnictwo_zwarte",
                "praca_doktorska",
                "praca_habilitacyjna",
                "patent",
            ],
        ),
        null=True,
        blank=True,
    )

    template = models.ForeignKey("dbtemplates.Template", on_delete=models.PROTECT)

    def __str__(self):
        if self.model_id is not None:
            return f"Powiązanie szablonu {self.template} z modelem {self.model}"
        return f"Powiązanie szablonu {self.template} z każdym modelem"

    class Meta:
        verbose_name = "powiązanie szablonu dla opisu bibliograficznego"
        verbose_name_plural = "powiązania szablonów dla opisu bibliograficznego"

    def render(self, praca):
        template = get_template(self.template.name)

        return (
            template.render(
                dict(praca=praca, autorzy=praca.autorzy_set.all().select_related())
            )
            .replace("\r\n", "")
            .replace("\n", "")
            .replace(".</b>[", ".</b> [")
            .replace("  ", " ")
            .replace("  ", " ")
            .replace("  ", " ")
            .replace("  ", " ")
            .replace("  ", " ")
            .replace(" , ", ", ")
            .replace(" . ", ". ")
            .replace(". . ", ". ")
            .replace(". , ", ". ")
            .replace("., ", ". ")
            .replace(" .", ".")
        )

    def get_models_for_this_szablon(self):
        return SzablonDlaOpisuBibliograficznego.objects.get_models_for_template(
            self.template
        )



# -*- encoding: utf-8 -*-

from django.db import models


class OpcjaWyswietlaniaField(models.CharField):
    POKAZUJ_ZAWSZE = "always"
    POKAZUJ_ZALOGOWANYM = "logged-in"
    POKAZUJ_NIGDY = "never"
    POKAZUJ_GDY_W_ZESPOLE = "staff"

    POKAZUJ_CHOICES = [
        (POKAZUJ_ZAWSZE, "zawsze"),
        (POKAZUJ_ZALOGOWANYM, "tylko dla zalogowanych"),
        (POKAZUJ_GDY_W_ZESPOLE, "tylko dla zespołu"),
        (POKAZUJ_NIGDY, "nigdy"),
    ]

    def __init__(
        self,
        verbose_name,
        default=POKAZUJ_ZAWSZE,
        max_length=50,
        choices=POKAZUJ_CHOICES,
        *args,
        **kw
    ):
        super(OpcjaWyswietlaniaField, self).__init__(
            verbose_name=verbose_name,
            max_length=max_length,
            choices=choices,
            default=default,
            *args,
            **kw
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-08 19:45
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0078_django110_py3k'),
    ]

    operations = [
        migrations.AddField(
            model_name='wydzial',
            name='skrot_nazwy',
            field=models.CharField(blank=True, help_text='Skrót nazwy wydziału, wersja czytelna, np. "Wydz. Lek."', max_length=250, null=True, unique=True),
        ),
    ]



from django.contrib.contenttypes.admin import GenericTabularInline

from bpp.models import Element_Repozytorium


class Element_RepozytoriumInline(GenericTabularInline):
    model = Element_Repozytorium
    extra = 0



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0062_auto_20161119_2207'),
    ]

    operations = [
        migrations.AddField(
            model_name='autor',
            name='aktualny',
            field=models.BooleanField(default=False, help_text=b'Je\xc5\xbceli zaznaczone, pole to oznacza,\n    \xc5\xbce autor jest aktualnie - na dzi\xc5\x9b dzie\xc5\x84 - przypisany do jakiej\xc5\x9b jednostki w bazie danych i jego przypisanie\n    do tej jednostki nie zosta\xc5\x82o zako\xc5\x84czone wraz z konkretn\xc4\x85 dat\xc4\x85 w przesz\xc5\x82o\xc5\x9bci.', verbose_name=b'Aktualny?'),
        ),
    ]



# -*- encoding: utf-8 -*-

from django.core.management import BaseCommand
from django.db import transaction

from bpp.models import (
    Wydawca,
    Wydawnictwo_Zwarte,
    Praca_Doktorska,
    Praca_Habilitacyjna,
    Wydawnictwo_Ciagle,
    cache,
)
import re

from bpp.util import wytnij_isbn_z_uwag


class Command(BaseCommand):
    help = 'Wycina ISBN z pola "Uwagi"'

    def add_arguments(self, parser):
        parser.add_argument("--przenies-do-uwag", action="store_true")
        parser.add_argument("--skasuj", action="store_true")

    @transaction.atomic
    def handle(self, przenies_do_uwag, skasuj, *args, **options):

        if przenies_do_uwag and skasuj:
            raise Exception("albo --skasuj, albo --przenies-do-uwag")

        if cache.enabled():
            cache.disable()

        for klass in Wydawnictwo_Zwarte, Wydawnictwo_Ciagle:
            for rec in klass.objects.filter(uwagi__istartswith="isbn"):

                res = wytnij_isbn_z_uwag(rec.uwagi)

                if res is None:
                    print(
                        f"*** No match for [{rec.uwagi}] ({rec.tytul_oryginalny}), why?"
                    )
                else:
                    isbn, rest = res

                    print(f"[{rec.uwagi}] => ISBN [{isbn}], uwagi [{rest}]")

                    if przenies_do_uwag:
                        rec.isbn = isbn
                        rec.uwagi = rest
                        rec.save()

                    if skasuj:
                        rec.uwagi = rest
                        rec.save()



# -*- encoding: utf-8 -*-
from optparse import make_option
import sys

from django.core.management import BaseCommand
from django.db import transaction
from django.test.utils import override_settings

from bpp.models.cache import Rekord
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
from bpp.models.zrodlo import Punktacja_Zrodla


class Command(BaseCommand):
    help = 'Ustawia punktacje dla niektorych typow prac'

    # option_list = BaseCommand.option_list + (
    #     make_option("--initial-offset", action="store", type="int", default=0),
    #     make_option("--skip", action="store", type="int", default=0),
    #     make_option("--pk", action="store", type="str", default=None)
    # )

    @transaction.atomic
    @override_settings(CELERY_ALWAYS_EAGER=True)
    def handle(self, *args, **options):
        # fn = ['impact_factor', 'punkty_kbn', 'index_copernicus', 'punktacja_wewnetrzna']
        fn = ['punkty_kbn',]

        for praca in Wydawnictwo_Ciagle.objects.filter(
            rok__gte=2016,
            charakter_formalny__skrot="AC",
            punkty_kbn__gt=0).only("pk", "punkty_kbn", "tytul_oryginalny", "zrodlo_id", "rok"):
            try:
                pz = Punktacja_Zrodla.objects.get(
                        rok=praca.rok,
                        zrodlo=praca.zrodlo_id)
            except Punktacja_Zrodla.DoesNotExist:
                print("BRAK PUNKTACJI ZRODLA: ", praca.zrodlo, praca.rok)
                continue

            ch = False
            for field in fn:
                v = getattr(praca, field)
                s = getattr(pz, field)
                if v != s:
                    setattr(praca, field, s)
                    ch = True
            if ch:
                print(praca.tytul_oryginalny)
                praca.save()


# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0024_auto_20150830_1705'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charakter_formalny',
            name='artykul_pbn',
            field=models.BooleanField(default=False, help_text=b'Wydawnictwa ci\xc4\x85g\xc5\x82e posiadaj\xc4\x85ce\n     ten charakter formalny zostan\xc4\x85 w\xc5\x82\xc4\x85czone do eksportu PBN jako artyku\xc5\x82y', verbose_name=b'Artyku\xc5\x82 w PBN'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='charakter_formalny',
            name='charakter_pbn',
            field=models.ForeignKey(on_delete=models.CASCADE, default=None, to='bpp.Charakter_PBN', blank=True, help_text=b'Warto\xc5\x9b\xc4\x87 wybrana w tym polu zostanie u\xc5\xbcyta jako zawarto\xc5\x9b\xc4\x87 tagu <is>\n                                      w plikach eksportu do PBN', null=True, verbose_name=b'Charakter PBN'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='charakter_formalny',
            name='ksiazka_pbn',
            field=models.BooleanField(default=False, help_text=b'Wydawnictwa zwarte posiadaj\xc4\x85ce ten\n    charakter formalny zostan\xc4\x85 w\xc5\x82\xc4\x85czone do eksportu PBN jako ksia\xc5\xbcki', verbose_name=b'Ksi\xc4\x85\xc5\xbcka w PBN'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='charakter_formalny',
            name='rozdzial_pbn',
            field=models.BooleanField(default=False, help_text=b'Wydawnictwa zwarte posiadaj\xc4\x85ce ten\n    charakter formalny zostan\xc4\x85 w\xc5\x82\xc4\x85czone do eksportu PBN jako rozdzia\xc5\x82y', verbose_name=b'Rozdzia\xc5\x82 w PBN'),
            preserve_default=True,
        ),
    ]



from denorm import CountField, denormalized, depend_on_related
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import JSONField

from bpp.fields import YearField
from bpp.models import ModelZNazwa, ModelZPBN_UID


class Wydawca(ModelZNazwa, ModelZPBN_UID):
    alias_dla = models.ForeignKey(
        "self", on_delete=models.CASCADE, null=True, blank=True
    )

    pbn_uid = models.ForeignKey(
        "pbn_api.Publisher",
        verbose_name="Odpowiednik w PBN",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )

    class Meta:
        verbose_name_plural = "wydawcy"
        verbose_name = "wydawca"
        ordering = ("nazwa",)

    def clean(self):
        if self.pk:
            if hasattr(self, "alias_dla_id") and self.pk == self.alias_dla_id:
                raise ValidationError("Rekord nie może być aliasem sam dla siebie")

            if getattr(self, "alias_dla_id") is not None:
                if self.poziom_wydawcy_set.count():
                    raise ValidationError(
                        "Wydawca ma przypisane poziomy wydawcy, "
                        "stąd nie można zmienić go w alias. "
                    )

    def save(self, *args, **kw):
        self.clean()
        return super().save(*args, **kw)

    def get_toplevel(self):
        if self.alias_dla_id is not None:
            return self.alias_dla.get_toplevel()
        return self

    def get_tier(self, rok):
        toplevel = self.get_toplevel()

        ret = toplevel.poziom_wydawcy_set.filter(rok=rok).first()
        if ret is None:
            return -1
        return ret.poziom

    def __str__(self):
        ret = self.nazwa
        if self.alias_dla_id is not None:
            ret += f" (alias dla {self.alias_dla.nazwa})"
        return ret

    # Poniższe zdenormalizowane pole zawiera wszystkie poziomy wydawcy, czyli będzie
    # aktualizowane w sytuacjach, gdy poziom wydawcy zostanie dodany lub usunięty:

    @denormalized(JSONField, blank=True, null=True)
    @depend_on_related("bpp.Poziom_Wydawcy")
    def lista_poziomow(self):
        return [
            (x.rok, x.poziom) for x in self.poziom_wydawcy_set.all().order_by("rok")
        ]

    # ile_aliasów to pole które liczy ilość wydawców mających tego wydawcę w polu "alias_dla"

    ile_aliasow = CountField("wydawca_set")


class Poziom_Wydawcy(models.Model):
    rok = YearField(db_index=True)
    wydawca = models.ForeignKey(Wydawca, models.CASCADE)
    poziom = models.PositiveSmallIntegerField(
        choices=[(None, "nieokreślono"), (1, "poziom I"), (2, "poziom II")],
        blank=True,
        null=True,
    )

    class Meta:
        unique_together = ("rok", "wydawca")
        verbose_name = "poziom wydawcy"
        verbose_name_plural = "poziomy wydawcy"
        ordering = ("rok",)

    def clean(self):
        if hasattr(self, "wydawca_id") and self.wydawca_id:
            if self.wydawca.alias_dla is not None:
                raise ValidationError(
                    "Nie można przypisywać poziomu wydawcy dla aliasów."
                )

    def save(self, *args, **kw):
        self.clean()
        return super().save(*args, **kw)

    def __str__(self):
        return f'Poziom wydawcy "{self.wydawca.nazwa}" za rok {self.rok}'



# -*- encoding: utf-8 -*-

# -*- encoding: utf-8 -*-

from django.contrib import admin

from bpp.models import Dyscyplina_Naukowa
from .core import RestrictDeletionToAdministracjaGroupMixin


class Dyscyplina_NaukowaAdmin(RestrictDeletionToAdministracjaGroupMixin, admin.ModelAdmin):
    list_display = ('kod', 'nazwa', 'dziedzina', 'widoczna')  # Sane defaults.

    fields = None
    search_fields = ('nazwa', 'kod',)
    list_filter = ('widoczna',)

    fields = ["nazwa", "kod", "widoczna", ]
    mptt_level_indent = 40


admin.site.register(Dyscyplina_Naukowa, Dyscyplina_NaukowaAdmin)



# Generated by Django 2.2.10 on 2020-03-04 21:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0199_alias_wydawcy_trigger"),
    ]

    operations = [
        migrations.AlterField(
            model_name="jednostka",
            name="skupia_pracownikow",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Ta jednostka skupia osoby będące faktycznymi pracownikami uczelni. Odznacz dla jednostek\n         typu 'Studenci', 'Doktoranci', 'Pracownicy emerytowani' itp. Odznacz dla 'obcych' jednostek. ",
                verbose_name="Skupia pracowników",
            ),
        ),
        migrations.AlterField(
            model_name="patent_autor",
            name="afiliuje",
            field=models.BooleanField(
                default=True,
                help_text="Afiliuje\n    się do jednostki podanej w przypisaniu. Jednostka nie może być obcą. ",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle_autor",
            name="afiliuje",
            field=models.BooleanField(
                default=True,
                help_text="Afiliuje\n    się do jednostki podanej w przypisaniu. Jednostka nie może być obcą. ",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte_autor",
            name="afiliuje",
            field=models.BooleanField(
                default=True,
                help_text="Afiliuje\n    się do jednostki podanej w przypisaniu. Jednostka nie może być obcą. ",
            ),
        ),
    ]



# Generated by Django 2.2.10 on 2020-03-15 19:21

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0202_auto_20200308_2204"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="cache_punktacja_dyscypliny", name="zapisani_wszyscy_autorzy",
        ),
        migrations.AddField(
            model_name="cache_punktacja_dyscypliny",
            name="autorzy_z_dyscypliny",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.PositiveIntegerField(),
                blank=True,
                null=True,
                size=None,
            ),
        ),
    ]



from django.db import models
from django.db.models import CASCADE

from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.contrib.contenttypes.models import ContentType

from django.utils.functional import cached_property

from bpp.models import Autor, Charakter_Formalny, DwaTytuly, ModelZOplataZaPublikacje
from bpp.models.praca_doktorska import Praca_Doktorska_Baza


class Publikacja_Habilitacyjna(models.Model):
    praca_habilitacyjna = models.ForeignKey("Praca_Habilitacyjna", CASCADE)
    kolejnosc = models.IntegerField("Kolejność", default=0)

    limit = (
        models.Q(app_label="bpp", model="wydawnictwo_ciagle")
        | models.Q(app_label="bpp", model="wydawnictwo_zwarte")
        | models.Q(app_label="bpp", model="patent")
    )
    content_type = models.ForeignKey(ContentType, CASCADE, limit_choices_to=limit)
    object_id = models.PositiveIntegerField()
    publikacja = GenericForeignKey()

    class Meta:
        app_label = "bpp"
        verbose_name = "powiązanie publikacji z habilitacją"
        verbose_name_plural = "powiązania publikacji z habilitacją"
        unique_together = [("praca_habilitacyjna", "content_type", "object_id")]
        ordering = ("kolejnosc",)


class _Praca_Habilitacyjna_PropertyCache:
    @cached_property
    def charakter_formalny(self):
        return Charakter_Formalny.objects.get(skrot="H")


_Praca_Habilitacyjna_PropertyCache = _Praca_Habilitacyjna_PropertyCache()


class Praca_Habilitacyjna(Praca_Doktorska_Baza):
    autor = models.OneToOneField(Autor, CASCADE)

    publikacje_habilitacyjne = GenericRelation(Publikacja_Habilitacyjna)

    @cached_property
    def charakter_formalny(self):
        return _Praca_Habilitacyjna_PropertyCache.charakter_formalny

    class Meta:
        verbose_name = "praca habilitacyjna"
        verbose_name_plural = "prace habilitacyjne"
        app_label = "bpp"

    def clean(self):
        DwaTytuly.clean(self)
        ModelZOplataZaPublikacje.clean(self)



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0053_obiekt_jednostka_pole_uczelnia'),
    ]

    operations = [
        migrations.AddField(
            model_name='jednostka',
            name='aktualna',
            field=models.BooleanField(default=False, help_text=b"Je\xc5\xbceli dana jednostka wchodzi w struktury wydzia\xc5\x82u\n    (czyli jej obecno\xc5\x9b\xc4\x87 w strukturach wydzia\xc5\x82u nie zosta\xc5\x82a zako\xc5\x84czona z okre\xc5\x9blon\xc4\x85 dat\xc4\x85), to pole to b\xc4\x99dzie mia\xc5\x82o\n    warto\xc5\x9b\xc4\x87 'PRAWDA'."),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-14 07:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0081_konferencje'),
    ]

    operations = [
        migrations.AlterField(
            model_name='konferencja',
            name='miasto',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Miasto'),
        ),
        migrations.AlterField(
            model_name='konferencja',
            name='panstwo',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Państwo'),
        ),
        migrations.AlterField(
            model_name='konferencja',
            name='rozpoczecie',
            field=models.DateField(blank=True, null=True, verbose_name='Rozpoczęcie'),
        ),
        migrations.AlterField(
            model_name='konferencja',
            name='skrocona_nazwa',
            field=models.CharField(blank=True, db_index=True, max_length=250, null=True, verbose_name='Skrócona nazwa'),
        ),
        migrations.AlterField(
            model_name='konferencja',
            name='zakonczenie',
            field=models.DateField(blank=True, null=True, verbose_name='Zakończenie'),
        ),
    ]



import pytest
from django.core.exceptions import ImproperlyConfigured, ValidationError
from model_bakery import baker

from bpp.models import Uczelnia


def test_Uczelnia_clean_pbn_biezaco_tak_integracja_nie(uczelnia):
    uczelnia.pbn_aktualizuj_na_biezaco = True
    uczelnia.pbn_integracja = False

    with pytest.raises(ValidationError, match="nie używasz integracji"):
        uczelnia.clean()


@pytest.mark.django_db
def test_Uczelnia_clean_integracja(uczelnia, mocker):
    uczelnia.pbn_integracja = True
    uczelnia.pbn_app_token = uczelnia.pbn_app_name = "hej"

    pbn_client = mocker.Mock()
    pbn_client.return_value.get_languages.side_effect = KeyError("foo")

    orig = uczelnia.pbn_client

    uczelnia.pbn_client = pbn_client

    with pytest.raises(ValidationError, match="Nie można pobrać"):
        uczelnia.clean()

    uczelnia.pbn_client = orig


@pytest.mark.django_db
def test_Uczelnia_pbn_client():
    uczelnia = baker.make(
        Uczelnia,
        nazwa="Testowa uczelnia",
        skrot="TU",
        pbn_app_name=None,
        pbn_app_token=None,
    )

    uczelnia.pbn_app_name = None
    uczelnia.save()

    uczelnia.refresh_from_db()

    try:
        res = uczelnia.pbn_client()
        raise Exception(
            f"This should not happen {uczelnia.pbn_app_name} {uczelnia} {uczelnia.pbn_client} {res}"
        )
    except ImproperlyConfigured:
        pass

    uczelnia.pbn_app_name = "foo"
    uczelnia.save()

    uczelnia.pbn_app_token = None
    uczelnia.save()

    try:
        res = uczelnia.pbn_client()
        raise Exception(
            f"This should not happen {uczelnia.pbn_app_name} {uczelnia} {uczelnia.pbn_client} {res}"
        )
    except ImproperlyConfigured:
        pass

    uczelnia.pbn_app_token = "foo"
    uczelnia.save()

    assert uczelnia.pbn_client()



from bpp.admin.actions import (
    ustaw_po_korekcie,
    ustaw_przed_korekta,
    ustaw_w_trakcie_korekty,
)
from bpp.models import Wydawnictwo_Zwarte


class FakeModelAdmin:
    def message_user(self, *args, **kw):
        pass


def test_ustaw_przed_korekta(wydawnictwo_zwarte_po_korekcie, przed_korekta):
    ustaw_przed_korekta(FakeModelAdmin(), None, Wydawnictwo_Zwarte.objects.all())
    wydawnictwo_zwarte_po_korekcie.refresh_from_db()
    assert wydawnictwo_zwarte_po_korekcie.status_korekty == przed_korekta


def test_ustaw_po_korekcie(wydawnictwo_zwarte_przed_korekta, po_korekcie):
    ustaw_po_korekcie(FakeModelAdmin(), None, Wydawnictwo_Zwarte.objects.all())
    wydawnictwo_zwarte_przed_korekta.refresh_from_db()
    assert wydawnictwo_zwarte_przed_korekta.status_korekty == po_korekcie


def test_ustaw_w_trakcie_korekty(wydawnictwo_zwarte_przed_korekta, w_trakcie_korekty):
    ustaw_w_trakcie_korekty(FakeModelAdmin(), None, Wydawnictwo_Zwarte.objects.all())
    wydawnictwo_zwarte_przed_korekta.refresh_from_db()
    assert wydawnictwo_zwarte_przed_korekta.status_korekty == w_trakcie_korekty



from raport_slotow.forms import ParametryRaportSlotowEwaluacjaForm


def test_ParametryRaporSlotowEwaluacjaForm_faulty():
    p = ParametryRaportSlotowEwaluacjaForm(
        {"od_roku": "2000", "do_roku": "2000", "_export": "xlsx"}
    )
    p.full_clean()
    assert len(p.errors) == 2


def test_ParametryRaporSlotowEwaluacjaForm_ok():
    p = ParametryRaportSlotowEwaluacjaForm(
        {"od_roku": "2017", "do_roku": "2018", "_export": "xlsx"}
    )
    p.full_clean()
    assert len(p.errors) == 0



import pytest
from django.urls import reverse

from bpp.models import OpcjaWyswietlaniaField
from formdefaults.models import FormRepresentation
from raport_slotow.views import (
    ParametryRaportSlotowEwaluacja,
    UtworzRaportSlotowUczelnia,
    WyborOsoby,
)


@pytest.mark.parametrize(
    "url,klass",
    [
        ("raport_slotow:index", WyborOsoby),
        ("raport_slotow:utworz-raport-slotow-uczelnia", UtworzRaportSlotowUczelnia),
        ("raport_slotow:index-ewaluacja", ParametryRaportSlotowEwaluacja),
    ],
)
def test_form_defaults_napis_przed_po(uczelnia, admin_client, url, klass):
    NAPIS_PRZED = b"napis przed"
    NAPIS_PO = b"napis po"

    uczelnia.pokazuj_raport_slotow_autor = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
    uczelnia.pokazuj_raport_slotow_uczelnia = OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
    uczelnia.save()

    res = admin_client.get(reverse(url))
    for n in NAPIS_PO, NAPIS_PRZED:
        assert n not in res.content

    form_class = klass().get_form_class()
    res = FormRepresentation.objects.get_or_create_for_instance(form_class())
    res.html_before = NAPIS_PRZED
    res.html_after = NAPIS_PO
    res.save()

    res = admin_client.get(reverse(url))
    for n in NAPIS_PO, NAPIS_PRZED:
        assert n in res.content



# Generated by Django 3.0.14 on 2021-07-13 20:35

from django.db import migrations

import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0015_auto_20210713_2234"),
    ]

    operations = [
        migrations.AlterField(
            model_name="publikacjainstytucji",
            name="snapshot",
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True),
        ),
    ]



import django_filters
from rest_framework import viewsets

from api_v1.serializers.praca_doktorska import Praca_DoktorskaSerializer
from api_v1.viewsets.common import UkryjStatusyKorektyMixin
from bpp.models import Praca_Doktorska


class Praca_DoktorskaFilterSet(django_filters.rest_framework.FilterSet):
    ostatnio_zmieniony = django_filters.DateTimeFromToRangeFilter("ostatnio_zmieniony")
    rok = django_filters.RangeFilter("rok")

    class Meta:
        fields = ["ostatnio_zmieniony", "rok"]
        model = Praca_Doktorska


class Praca_DoktorskaViewSet(UkryjStatusyKorektyMixin, viewsets.ReadOnlyModelViewSet):
    # Lista musi być posortowana po PK aby nie było duplikatów
    queryset = (
        Praca_Doktorska.objects.exclude(nie_eksportuj_przez_api=True)
        .order_by("pk")
        .select_related("status_korekty")
        .prefetch_related("slowa_kluczowe")
    )
    serializer_class = Praca_DoktorskaSerializer
    filterset_class = Praca_DoktorskaFilterSet



# Generated by Django 3.0.14 on 2021-06-06 12:29

from django.db import migrations, models

import bpp.models.dyscyplina_naukowa


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0272_auto_20210605_2128"),
    ]

    operations = [
        migrations.AddField(
            model_name="jednostka",
            name="level",
            field=models.PositiveIntegerField(default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="jednostka",
            name="lft",
            field=models.PositiveIntegerField(default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="jednostka",
            name="rght",
            field=models.PositiveIntegerField(default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="jednostka",
            name="tree_id",
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="dyscyplina_naukowa",
            name="kod",
            field=bpp.models.dyscyplina_naukowa.KodDyscyplinyField(
                max_length=20,
                unique=True,
                validators=[
                    bpp.models.dyscyplina_naukowa.waliduj_format_kodu_numer,
                    bpp.models.dyscyplina_naukowa.waliduj_format_kodu_numer,
                    bpp.models.dyscyplina_naukowa.waliduj_format_kodu_numer,
                ],
            ),
        ),
    ]



# Generated by Django 3.0.11 on 2021-01-25 01:56

import django.db.models.deletion
from django.db import migrations, models

import bpp.models.uczelnia


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0235_auto_20210125_0042"),
        ("raport_slotow", "0004_raportslotowuczelnia_raportslotowuczelniawiersz"),
    ]

    operations = [
        migrations.AddField(
            model_name="raportslotowuczelniawiersz",
            name="avg",
            field=models.DecimalField(
                decimal_places=4,
                default=0,
                max_digits=8,
                verbose_name="Średnio punktów dla autora na slot",
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="raportslotowuczelnia",
            name="do_roku",
            field=models.IntegerField(
                default=bpp.models.uczelnia.UczelniaManager.do_roku_default
            ),
        ),
        migrations.AlterField(
            model_name="raportslotowuczelniawiersz",
            name="jednostka",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="bpp.Jednostka",
            ),
        ),
    ]



# Generated by Django 2.1.7 on 2019-08-24 05:37

from django.db import migrations, models

from bpp import const


def wypelnij_rodzaj_pbn(apps, schema_editor):
    Charakter_Formalny = apps.get_model("bpp", "Charakter_Formalny")
    for cf in Charakter_Formalny.objects.all():
        needs_save = False
        if cf.artykul_pbn:
            cf.rodzaj_pbn = const.RODZAJ_PBN_ARTYKUL
            needs_save = True
        if cf.rozdzial_pbn:
            cf.rodzaj_pbn = const.RODZAJ_PBN_ROZDZIAL
            needs_save = True
        if cf.ksiazka_pbn:
            cf.rodzaj_pbn = const.RODZAJ_PBN_KSIAZKA
            needs_save = True
        if needs_save:
            cf.save()


class Migration(migrations.Migration):
    dependencies = [
        ("bpp", "0173_auto_20190812_1412"),
    ]

    operations = [
        migrations.AddField(
            model_name="charakter_formalny",
            name="rodzaj_pbn",
            field=models.PositiveSmallIntegerField(
                blank=True,
                choices=[
                    (None, "nie eksportuj do PBN"),
                    (1, "artykuł"),
                    (3, "książka"),
                    (2, "rozdział"),
                ],
                default=None,
                help_text="Pole określające, czy wydawnictwa posiadające dany charakter formalny zostaną włączone\n        do eksportu PBN jako artykuły, rozdziały czy książki. ",
                null=True,
                verbose_name="Rodzaj dla PBN",
            ),
        ),
        migrations.RunPython(wypelnij_rodzaj_pbn),
        migrations.RemoveField(
            model_name="charakter_formalny",
            name="artykul_pbn",
        ),
        migrations.RemoveField(
            model_name="charakter_formalny",
            name="ksiazka_pbn",
        ),
        migrations.RemoveField(
            model_name="charakter_formalny",
            name="rozdzial_pbn",
        ),
    ]



# Generated by Django 3.0.9 on 2020-10-10 22:02

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0225_auto_20201007_1002"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0226_rekord_mat_status")
        ),
        # Odbudowanie tabeli rekord_mat spowoduje skasowanie bpp_uczelnia_ewaluacja_view,
        # stąd potrzeba zaciągnąć ten plik z poprzednich migracji:
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0207_uczelnia_analiza_view")
        ),
    ]
